/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/conf2.h: run-time configuration header file
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: conf2.h,v 1.87 2004/05/13 22:26:56 rageircd Exp $
 */

#ifndef __conf2_include__
#define __conf2_include__ 

#include "setup.h"
#include "dlink.h"
#include <time.h>
#include <sys/types.h>
#include <netinet/in.h>

struct dns_query;
struct auth;
struct Client;
struct listener;

#define MAXCONFHOSTS 16

typedef struct _configfile ConfigFile;
typedef struct _configentry ConfigEntry;
typedef struct _configunknown ConfigUnknown;
typedef struct _configdirective ConfigDirective;
typedef struct _configblock ConfigBlock;
typedef struct _configoption ConfigOption;
typedef struct _configfrom ConfigFrom;
typedef struct _configitem ConfigItem;

typedef struct _configitem_servinfo ConfigItem_servinfo;
typedef struct _configitem_admin ConfigItem_admin;
typedef struct _configitem_class ConfigItem_class;
typedef struct _configitem_allow ConfigItem_allow;
typedef struct _configitem_oper ConfigItem_oper;
typedef struct _configitem_link ConfigItem_link;
typedef struct _configitem_listen ConfigItem_listen;
typedef struct _configitem_ban ConfigItem_ban;
typedef struct _configitem_except ConfigItem_except;
typedef struct _configitem_vhost ConfigItem_vhost;
typedef struct _configitem_include ConfigItem_include;

typedef struct _configitem_network ConfigItem_network;
typedef struct _configitem_masking ConfigItem_masking;
typedef struct _configitem_flood ConfigItem_flood;
typedef struct _configitem_general ConfigItem_general;

struct _configfile {
	ConfigFile *next;
	char *filename;
	ConfigEntry *entries;
};

struct _configentry {
	ConfigEntry *entries, *next, *prev;
	ConfigFile *file;
	int varlinenum, sectlinenum;
	char *varname;
	char *vardata;
};

struct _configunknown {
	ConfigEntry *entry;
	void *item;
	short type;
};

struct _configdirective {
	char *directive;
	void *(*parse_func)(ConfigEntry *);
	int (*test_func)(ConfigFile *, void *);
	void (*free_func)(void *);
};

struct _configblock {
	ConfigFile *file;
	ConfigDirective *dir;
	void *item;
};

struct _configoption {
	char *token;
	unsigned int flag;
};

struct _configfrom {
	char *hosts[MAXCONFHOSTS + 1];
	int host_count;
};

struct _configitem {
	unsigned temp : 1;
	unsigned perm : 1;
	int refcount; /* How many times this item is referenced */
};

struct _configitem_servinfo {
	char *name;
	char *desc;
#ifdef USE_OPENSSL
	char *ssl_certificate;
	char *ssl_private_key;
#endif
	char *default_bind_ip;
	unsigned int identity;
	char *kline_address;
	int max_clients;
	unsigned int logs;
	unsigned hub : 1;
	char *resolver_conf_file;
};

struct _configitem_admin {
	char *name;
	char *desc;
	char *email;
};

struct _configitem_class {
	ConfigItem item;
	char *name;
	time_t ping_time;
	int max_clients;
	long sendq_length;
	int clients;
	unsigned int flags;
};

struct _configitem_allow {
	ConfigItem item;
	char *hostname;
	char *ipaddr;
	unsigned hostname_at : 1;
	unsigned ipaddr_at : 1;
	int port;
	int max_per_ip;
	ConfigItem_class *class;
	struct auth *auth;
	char *redir_serv;
	int redir_port;
};

struct _configitem_oper {
	ConfigItem item;
	char *name;
	struct auth *auth;
	ConfigFrom from;
	ConfigItem_class *class;
	char *join_on_oper;
	unsigned int flags;
};

struct _configitem_link {
	ConfigItem item;
	char *servername;
	char *host;
	char *bind_ip;
	struct auth *auth;
	int port;
	ConfigItem_class *class;
	unsigned int flags;
	int servers;
	struct in_addr ip;
	time_t next_connect;
	struct dns_query *dns_query;
};

struct _configitem_listen {
	ConfigItem item;
	char *ipaddr;
	int port;
	unsigned int flags;
	struct listener *l;
};

struct _configitem_ban {
	ConfigItem item;
	unsigned kill_on_rehash : 1;
	unsigned in_file : 1;
	char *mask;
	char *reason;
	time_t expire;
	time_t set_at;
	unsigned int flags;
};

struct _configitem_except {
	char *mask;
	unsigned int flags;
};

struct _configitem_vhost {
	char *mask;
	char *login;
	struct auth *auth;
	ConfigFrom from;
};

struct _configitem_include {
	char *filename;
	unsigned loaded : 1;
};

struct _configitem_network {
	char *name;
	char *name_005;
	char *kline_address;
	char *services_server;
	char *stats_server;
	int max_link_depth;
};

struct _configitem_masking {
	char *netadmin_mask;
	char *admin_mask;
	char *oper_mask;
	char *user_mask_prefix;
	long mask_key1;
	long mask_key2;
	long mask_key3;
	unsigned auto_mask_opers : 1;
};

struct _configitem_flood {
	int msg_flood_count;
	time_t knock_delay;
	time_t accept_notice_time;
	time_t antispam_quit_msg_time;
	unsigned anti_nick_flood : 1;
	unsigned anti_away_flood : 1;
	unsigned no_oper_flood : 1;
	time_t pace_wait_simple;
	time_t pace_wait_intense;
#ifdef USE_THROTTLE
	time_t min_connect_time;
	int max_connect_count;
	unsigned throttle_rejected_clients : 1;
#endif
};

struct _configitem_general {
	int max_chans_per_user;
	time_t connauth_timeout;
	time_t statsfile_save_freq;
	unsigned int modes_on_connect;
	char *join_on_connect;
	int compression_level;
	time_t auto_connect_freq;
	time_t ts_delta_warn;
	time_t ts_delta_max;

	unsigned short_motd : 1;
	unsigned hide_ulined_servers : 1;
	unsigned check_identd : 1;
	unsigned show_headers : 1;
	unsigned failed_oper_notice : 1;
	unsigned enable_knock : 1;
	unsigned custom_quit_msgs : 1;
	unsigned show_invisible_lusers : 1;
	unsigned write_perm_klines_to_file : 1;
	unsigned flatten_links : 1;
	unsigned enable_map : 1;
	unsigned spy_notices : 1;
	unsigned resolve_hostnames : 1;
	unsigned custom_channels : 1;
	unsigned show_cliconn_quit_msgs : 1;
	unsigned no_oper_accept : 1;
};

enum {
	CONF_SERVINFO,
	CONF_ADMIN,
	CONF_CLASS,
	CONF_ALLOW,
	CONF_OPER,
	CONF_LINK,
	CONF_BAN,
	CONF_NETWORK,
	CONF_MASKING,
	CONF_FLOOD,
	CONF_GENERAL
};

enum {
	CONF_FALLBACK = -2,
	CONF_FAILURE = -1,
	CONF_NOMATCH,
	CONF_SUCCESS
};

#define LOG_DEFAULT			0x0001
#define LOG_ERROR			0x0002
#define LOG_KILL			0x0004
#define LOG_BAN				0x0008
#define LOG_CLIENT			0x0010
#define LOG_SERVER			0x0020
#define LOG_OPER			0x0040
#define LOG_DEBUG			0x0080
#define LOG_OVERRIDE			0x0100

#define CLASS_IGNOREIDENTD		0x0001
#define CLASS_REQUIREIDENTD		0x0002
#define CLASS_NOTILDE			0x0004

#define LINK_AUTOCONNECT		0x0001
#define LINK_COMPRESS			0x0002
#ifdef USE_OPENSSL
#define LINK_SECURE			0x0004
#endif

#define LISTEN_BOUND			0x0001
#define LISTEN_CLIENTONLY		0x0002
#define LISTEN_SERVERONLY		0x0004
#ifdef USE_OPENSSL
#define LISTEN_SECURE			0x0008
#endif
#define LISTEN_CRJAVA			0x0010

#define BAN_SVS				0x0001
#define BAN_USERHOST			0x0002
#define BAN_IP				0x0004
#define BAN_NICKNAME			0x0008
#define BAN_GCOS			0x0010
#define BAN_SERVER			0x0020
#define BAN_FILE			0x0040
#define BAN_CHANNEL			0x0080

extern dlink_list conf_file_list;
extern dlink_list conf_block_list;
extern dlink_list conf_unknown_list;

extern ConfigItem_network NetworkConfig, tmpNetworkConfig;
extern ConfigItem_masking MaskingConfig, tmpMaskingConfig;
extern ConfigItem_flood FloodConfig, tmpFloodConfig;
extern ConfigItem_general GeneralConfig, tmpGeneralConfig;

extern ConfigItem_servinfo *ServerInfo;
extern ConfigItem_admin *AdminInfo;
extern ConfigItem_class *DefaultClass;

extern dlink_list conf_class_list;
extern dlink_list conf_allow_list;
extern dlink_list conf_oper_list;
extern dlink_list conf_uline_list;
extern dlink_list conf_link_list;
extern dlink_list conf_listen_list;
extern dlink_list conf_ban_list;
extern dlink_list conf_except_list;
extern dlink_list conf_vhost_list;
extern dlink_list conf_include_list;

extern ConfigDirective conf_directives[];
extern ConfigOption servinfo_logs[];
extern ConfigOption oper_can_perform[];
extern ConfigOption oper_can_see[];
extern ConfigOption ban_types[];

extern void *parse_servinfo(ConfigEntry *);
extern void *parse_admin(ConfigEntry *);
extern void *parse_class(ConfigEntry *);
extern void *parse_allow(ConfigEntry *);
extern void *parse_oper(ConfigEntry *);
extern void *parse_uline(ConfigEntry *);
extern void *parse_link(ConfigEntry *);
extern void *parse_listen(ConfigEntry *);
extern void *parse_ban(ConfigEntry *);
extern void *parse_except(ConfigEntry *);
extern void *parse_vhost(ConfigEntry *);
extern void *parse_include(ConfigEntry *);
extern void *parse_modules(ConfigEntry *);
extern void *parse_network(ConfigEntry *);
extern void *parse_masking(ConfigEntry *);
extern void *parse_flood(ConfigEntry *);
extern void *parse_general(ConfigEntry *);

extern int test_servinfo(ConfigFile *, void *);
extern int test_class(ConfigFile *, void *);
extern int test_allow(ConfigFile *, void *);
extern int test_oper(ConfigFile *, void *);
extern int test_link(ConfigFile *, void *);
extern int test_ban(ConfigFile *, void *);
extern int test_vhost(ConfigFile *, void *);
extern int test_network(ConfigFile *, void *);
extern int test_masking(ConfigFile *, void *);
extern int test_general(ConfigFile *, void *);

extern void free_servinfo(void *);
extern void free_class(void *);
extern void free_allow(void *);
extern void free_oper(void *);
extern void free_link(void *);
extern void free_bind(void *);
extern void free_ban(void *);
extern void free_vhost(void *);
extern void free_network(void *);
extern void free_masking(void *);
extern void free_general(void *);

extern ConfigItem_class *find_class(char *);
extern ConfigItem_oper *find_oper(char *);
extern char *find_uline(char *);
extern ConfigItem_link *find_link(char *, char *);
extern ConfigItem_listen *find_listen(char *, int);
extern ConfigItem_ban *find_ban(unsigned int, char *);
extern ConfigItem_except *find_except(unsigned int, char *);

extern void report(short, char *, ...);
extern void init_conf();
extern void conf_rehash();
extern int conf_do_kline(ConfigItem_ban *, struct Client *, struct Client *);
extern int conf_do_gline(ConfigItem_ban *, struct Client *, struct Client *);
extern int conf_do_zap(ConfigItem_ban *, char *, int);
extern int conf_check_from(ConfigFrom *, char *);

extern int attach_allow(struct Client *, ConfigItem_allow *);
extern void detach_allow(struct Client *);
extern void attach_class(struct Client *);
extern void detach_class(struct Client *);
extern void attach_oper(struct Client *, ConfigItem_oper *);
extern void detach_oper(struct Client *);
extern void attach_link(struct Client *, ConfigItem_link *);
extern void detach_link(struct Client *);
extern void attach_listen(struct listener *, ConfigItem_listen *);
extern void detach_listen(struct listener *);
extern void detach_confs(struct Client *);

#define cDupString(xx, yy)	if ((xx) != NULL) MyFree((xx)); DupString((xx), (yy))
#define cMyFree(xx)		if ((xx) != NULL) MyFree((xx)); (xx) = NULL

#define EACH_ENTRY(xx, yy)	(xx) = (yy)->entries; (xx) != NULL; (xx) = (xx)->next

/* Check we have a block name, or return NULL */
#define BLOCK_NAME(xx, yy, zz)	if (BadPtr((xx)->vardata)) { report(0, "%s:%d: %s: missing %s", \
					(xx)->file->filename, (xx)->varlinenum, (yy), (zz)); \
				return NULL; }

/* Check block has entries, or return NULL */
#define BLOCK_ENTRIES(xx, yy)	if ((xx)->entries == NULL) { report(0, "%s:%d: %s: empty block", \
					(xx)->file->filename, (xx)->varlinenum, (yy)); \
				return NULL; }

/* Check section has entries, or continue */
#define SECTION(xx, yy)		if ((xx)->entries == NULL) { report(0, "%s:%d: %s: empty section", \
					(xx)->file->filename, (xx)->varlinenum, (yy)); \
				continue; }

/* Check variable, or continue */
#define VARIABLE(xx, yy)	if (BadPtr((xx)->varname)) { report(0, "%s:%d: %s: missing variable name", \
					(xx)->file->filename, (xx)->varlinenum, (yy)); \
				continue; }

/* Check parameter, or continue */
#define PARAMETER(xx, yy)	if (BadPtr((xx)->vardata)) { report(0, "%s:%d: %s: missing variable parameter", \
					(xx)->file->filename, (xx)->varlinenum, (yy)); \
				continue; }

#define UNKNOWN_VAR(xx, yy)	report(0, "%s:%d: %s: unknown variable %s", (xx)->file->filename, \
					(xx)->varlinenum, (yy), (xx)->varname)

#define UNKNOWN_VARNA(xx)	report(0, "%s:%d: unknown variable %s", (xx)->file->filename, \
					(xx)->varlinenum, (xx)->varname)

#endif /* __conf2_include__ */
