/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/config.h: compile time configuration header file
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: config.h,v 1.123 2004/05/13 18:15:30 rageircd Exp $
 */

#ifndef	__config_include__
#define	__config_include__

#include "setup.h"

#define LOWEST_FD	4 /* skip stdin, stdout, stderr, and profiler */

#define HARD_FDLIMIT	(HARD_FDLIMIT_ - 10)
#define MASTER_MAX	(HARD_FDLIMIT - MAX_BUFFER)

#ifdef OS_SUNOS_DBE
#define FD_SETSIZE HARD_FDLIMIT_
#define NOFILE HARD_FDLIMIT_
#endif

/*
 * Files and Directories
 * IRCD_PREFIX is set using ./configure --prefix=PREFIX
 */
#ifdef OS_CYGWIN
#define IRCD_EXE		IRCD_PREFIX "/ircd"
#else
#define IRCD_EXE		IRCD_PREFIX "/bin/ircd"
#endif
#define CONFIG_FILE		"ircd.conf"
#define MOTD_FILE		"ircd.motd"
#define OPERMOTD_FILE		"ircd.opermotd"
#define RULES_FILE		"ircd.rules"
#define STATS_FILE		"ircd.stats"
#define PID_FILE		"ircd.pid"
#define ENTROPY_FILE		"ircd.entropy"

#define LOGFILE_DEFAULT		"logs/ircd.log"
#define LOGFILE_ERROR		"logs/error.log"
#define LOGFILE_KILL		"logs/kill.log"
#define LOGFILE_BAN		"logs/ban.log"
#define LOGFILE_CLIENT		"logs/client.log"
#define LOGFILE_SERVER		"logs/server.log"
#define LOGFILE_OPER		"logs/oper.log"
#define LOGFILE_DEBUG		"logs/debug.log"
#define LOGFILE_OVERRIDE	"logs/override.log"

/*
 * Service Client Nicknames
 *
 * These are used for the service alias functions, and should be changed
 * according to the services available on your network. These were
 * dynamic configuration options, however, I fail to see why anyone
 * would want to change them anyway.
 */
#define CHANSERV "ChanServ"
#define NICKSERV "NickServ"
#define MEMOSERV "MemoServ"
#define OPERSERV "OperServ"
#define STATSERV "StatServ"
#define HOSTSERV "HostServ"
#define BOTSERV "BotServ"

/*
 * Memory Pre-allocation
 *
 * These determine how many structures memory should be pre-allocated for
 * at bootup, used with the block allocator.
 */
#define CLIENTS_PREALLOCATE		256	/* aClient */
#define LOCALCLIENTS_PREALLOCATE	128	/* LocalClient */
#define USERS_PREALLOCATE		256	/* anUser */
#define LOCALUSERS_PREALLOCATE		128	/* LocalUser */

#define SLINKS_PREALLOCATE 64
#define DLINKNODES_PREALLOCATE 256
#define CHANMEMBERS_PREALLOCATE 256
#define CHANNELBANS_PREALLOCATE 256
#define CHANNELS_PREALLOCATE 128
#define THROTTLES_PREALLOCATE 128
#define HASHENTS_PREALLOCATE 128
#define CONNAUTH_PREALLOCATE 64

/* INITIAL_SBUF_x -- how many bytes of shared buffers to preallocate */
#define INITIAL_SBUF_SMALL (1 << 20)		/* 1mb */
#define INITIAL_SBUF_LARGE (1 << 20)		/* 1mb */

/* INITIAL_SBUF_USER_POOL
 * How many user shared buffers to pre-allocate and pool.
 */
#define INITIAL_SBUF_USER_POOL		LOCALCLIENTS_PREALLOCATE

/*
 * Throttle Support
 *
 * THROTTLE_RECORDTIME - length to keep records for each ip (since last connect)
 * THROTTLE_HASHSIZE   - size of the throttle hash table
 *
 * We recommend you do not change these unless it's crutial.
 */
#define THROTTLE_RECORDTIME 1800
#define THROTTLE_HASHSIZE 25147

/*
 * BLOCKHEAP_CLEANUP_FREQ
 *
 * If the block heap allocator is enabled, this is the time in seconds between
 * automatic memory garbage collection, which clean up block heaps to eliminate
 * fragmentation and memory leaks.
 *
 * Default is 3 hrs.
 */
#define BLOCKHEAP_CLEANUP_FREQ (60 * 60) * 3

/*
 * TRY_CONNECTIONS_FREQ
 *
 * The number of seconds between calls to try_connections(), to auto-connect
 * to server links. Don't change this unless you know what you're doing.
 * Doing it too early may cause a lot of nick collisions.
 */
#define TRY_CONNECTIONS_FREQ 60

/*
 * MODULE_DELAY
 *
 * Sets the number of seconds for ircd to wait before attempting to load
 * delayed modules (caused by unresolved symbol dependencies)
 *
 * MODULE_DELAY_RETRIES
 *
 * Amount of times to retry loading delayed modules before considering
 * the broken dependency to be permanent.
 */
#define MODULE_DELAY 10
#define MODULE_DELAY_RETRIES 2

#include "defs.h"

/*
 * ABORT_ON_MALLOC_FAILURE
 *
 * If defined, ircd will abort (core dump) in the event of memory
 * allocation failure. This will allow for debugging, but results
 * in the server being terminated if system memory is low or non-
 * existant. Not recommended for production servers, only debugging.
 */
#define ABORT_ON_MALLOC_FAILURE

/*
 * INCREASE_SOCK_BUFS
 *
 * If defined, the size of the send and receive socket buffers will be
 * dynamically allocated upon connection. Server buffers will then be
 * increased to 64k. However, client buffers shall remain at 8k.
 */
#define INCREASE_SOCK_BUFS

/*
 * IRCII_KLUDGE
 *
 * Define this will accomplish ircII standards by sending an extra
 * NOTICE at the beginning of client connections.
 */
#undef IRCII_KLUDGE

/*
 * CLIENT_FLOOD
 *
 * This controls the number of bytes the server will allow a client
 * to send to the server without processing before the client is
 * dropped for flooding. Values greater than 8000 make no difference
 * to the server.
 */
#define	CLIENT_FLOOD 3500

/*
 * ANTI_NICK_FLOOD_CHANGES and ANTI_NICK_FLOOD_TIME
 *
 * These are the parameters for nickname changing flood protection.
 * Provided general::options::anti-nick-flood is enabled, any nickname
 * changes will be blocked if the client in question has performed
 * ANTI_NICK_FLOOD_CHANGES within ANTI_NICK_FLOOD_TIME seconds.
 */
#define ANTI_NICK_FLOOD_CHANGES 4
#define ANTI_NICK_FLOOD_TIME 20

/*
 * MAX_AWAY_COUNT and MAX_AWAY_TIME
 *
 * These provide the parameters for the AWAY flood protection.
 * Provided servinfo::options::anti-away-flood is enabled, any client
 * attempting to do MAX_AWAY_COUNT AWAY message changes within
 * MAX_AWAY_TIME seconds will be ignored.
 */
#define MAX_AWAY_COUNT 5
#define MAX_AWAY_TIME 180

/*
 * ALWAYS_SEND_DURING_SPLIT
 *
 * on a large network, if your server is carrying large amounts of clients,
 * and your server splits from the main network, the amount of allocated
 * dbufs will skyrocket as buffers fill up with QUIT messages. This code
 * attempts to combat this by sending out data whenever possible during a
 * split. - lucas
 */
#define ALWAYS_SEND_DURING_SPLIT

/*
 * CMDLINE_CONFIG
 *
 * Define this to enable the -c command-line option to specify an alternate
 * ircd.conf file.
 */
#define CMDLINE_CONFIG

/*
 * MAXCONNECTIONS
 *
 * WARNING: Do not change this directly. Change the HARD_FDLIMIT_
 * variable instead.
 *
 * Maximum number of network connections your server will allow.  This
 * should never exceed max. number of open file descrpitors and wont
 * increase this. Should remain LOW as possible. Most sites will
 * usually have under 30 or so connections. A busy hub or server may
 * need this to be as high as 50 or 60. Making it over 100 decreases
 * any performance boost gained from it being low. if you have a lot of
 * server connections, it may be worth splitting the load over 2 or
 * more servers. 1 server = 1 connection, 1 user = 1 connection. This
 * should be at *least* 3: 1 listen port, 1 dns port + 1 client
 */
#define MAXCONNECTIONS	HARD_FDLIMIT

/*
 * NICKNAMEHISTORYLENGTH
 *
 * This defines the length of the nickname history.  Each time a user
 * changes nickname or signs off, their old nickname is added to the
 * top of the list.
 *
 * WARNING: This is directly related to the amount of memory ircd will
 * use whilst resident and running - it hardly ever gets swapped to
 * disk! Memory will be preallocated for the entire whowas array when
 * ircd is started.
 */
#define NICKNAMEHISTORYLENGTH 1024

/*
 * TIMESEC - Time interval to wait and if no messages have been
 * received, then check for PINGFREQUENCY and CONNECTFREQUENCY
 */
#define TIMESEC 5

/*
 * DEFAULT_COMPRESSION_LEVEL
 *
 * Default level of compression to use for compressed server<->server
 * links. Range from 0 to 9, 0 being disabled. The higher the level,
 * the more compression is performed, but the more cpu time is used.
 */
#define DEFAULT_COMPRESSION_LEVEL 4

/*
 * DEFAULT_TS_DELTA_MAX and DEFAULT_TS_DELTA_WARN
 *
 * Default values for general::ts_delta_max and ts_delta_warn. If the
 * difference between our system time, and the time of a remote link
 * exceeds ts_delta_max, drop the link. If it exceeds ts_delta_warn,
 * send a routing notice.
 */
#define DEFAULT_TS_DELTA_MAX 120
#define DEFAULT_TS_DELTA_WARN 60

/*
 * DEFAULT_NETWORK_NAME, DEFAULT_NETWORK_NAME_005
 *
 * Default IRC network names, should none be specified in ircd.conf. Note
 * the latter should be identical to the former, but replace spaces with
 * a dash or underscore. (eg, "IRC Network" becomes "IRC-Network".)
 */
#define DEFAULT_NETWORK_NAME "IRC"
#define DEFAULT_NETWORK_NAME_005 "IRC"

/*
 * DEFAULT_KLINE_ADDRESS
 *
 * Default kline address to use, should none be specified in ircd.conf.
 * This will be used for both local and remote bans (ie, network-wide
 * and local-only).
 */
#define DEFAULT_KLINE_ADDRESS "admin@poorly.configured.server"

/*
 * DEFAULT_USER_MASK_PREFIX
 *
 * Default string to prefix user masked hostnames, should none be
 * specified in ircd.conf. This is the string that appears before
 * the "-CRC32HEX.isp.com" in user masked hostnames.
 */
#define DEFAULT_USER_MASK_PREFIX "masked"

/*
 * DEFAULT_KLINE_EXPIRE_TIME
 *
 * Default time (in seconds) to set K:Line expire times to if
 * servinfo::options::write-perm-klines-to-file is enabled, but if
 * the server fails to open CONFIG_FILE.
 */
#define DEFAULT_KLINE_EXPIRE_TIME (3600 * 24)

/*
 * DEFAULT_PINGTIME
 *
 * Default amount of seconds before the server checks internal list
 * to see if any connections that have been inactive for a specified
 * amount of time are still alive.
 */
#define DEFAULT_PINGTIME 120

/*
 * DEFAULT_CHANS_PER_USER
 *
 * Default amount of channels a client is allowed to be in at any
 * one time.
 */
#define DEFAULT_CHANS_PER_USER 10

/*
 * DEFAULT_MAX_CLIENTS
 *
 * Default amount of connections allowed in a class at any one time.
 */
#define DEFAULT_MAX_CLIENTS 256

/*
 * DEFAULT_LINK_DEPTH
 *
 * Default depth of a server link.
 */
#define DEFAULT_LINK_DEPTH 5

/*
 * DEFAULT_CONNAUTH_TIMEOUT
 *
 * Default amount of time, in seconds, to cycle the internal resolver
 * list to expire un-resolved hosts.
 */
#define DEFAULT_CONNAUTH_TIMEOUT 3

/*
 * DEFAULT_PORTNUM
 *
 * This defines the default port number to use in configuration if the
 * specified port is out of range. Range is from 1 to 65535.
 */
#define DEFAULT_PORTNUM 7000

/*
 * HANGONGOODLINK and HANGONGOODLINK
 *
 * Often net breaks for a short time and it's useful to try to 
 * establishing the same connection again faster than DEFAULT_CONNECTFREQ
 * would allow. But, to keep trying on bad connection, we require that 
 * connection has been open for certain minimum time (HANGONGOODLINK) and 
 * we give the net few seconds to steady (HANGONRETRYDELAY). This latter 
 * has to be long enough that the other end of the connection has time to 
 * notice it broke too.
 */

#define HANGONRETRYDELAY 60
#define HANGONGOODLINK 3600

/*
 * WRITEWAITDELAY
 *
 * Number of seconds to wait for write to complete if stuck.
 */
#define WRITEWAITDELAY     10

/*
 * CONNECTTIMEOUT
 *
 * Number of seconds to wait for a connect(2) call to complete.
 *
 * NOTE: This must be at LEAST 10. When a client connects, it has
 * CONNECTTIMEOUT - 10 seconds for its host to respond to an identd
 * lookup and for a DNS answer to be retrieved.
 */
#define	CONNECTTIMEOUT	30

/*
 * KILLCHASETIMELIMIT
 *
 * Max time from the nickname change that still causes KILL automaticly 
 * to switch for the current nick of that user.
 */
#define KILLCHASETIMELIMIT 90

/*
 * FLUD - CTCP Flood Detection and Protection
 * 
 * This enables server CTCP flood detection and protection for local
 * clients. It works well against fludnets and flood clones.  The
 * effect of this code on server CPU and memory usage is minimal,
 * however you may not wish to take the risk, or be fundamentally
 * opposed to checking the contents of PRIVMSG's (though no privacy is
 * breached).  This code is not useful for routing only servers (ie,
 * HUB's with little or no local client base), and the hybrid team
 * strongly recommends that you do not use FLUD with HUB. The following
 * default thresholds may be tweaked, but these seem to work well.
 */
#define FLUD

#ifdef FLUD
#define FLUD_NUM 4	/* Number of flud messages to trip alarm */
#define FLUD_TIME 3	/* Seconds in which FLUD_NUM msgs must occur */
#define FLUD_BLOCK 15	/* Seconds to block fluds */
#endif

/*
 * ANTI_SPAMBOT
 *
 * If ANTI_SPAMBOT is defined try to discourage spambots. The defaults
 * should be fine for the timers/counters etc. but you can play with
 * them. -Dianora
 *
 * Defining this also does a quick check whether the client sends us a
 * "user foo x x :foo" where x is just a single char.  More often than
 * not, it's a bot if it did. -ThemBones
 */
#define ANTI_SPAMBOT

/*
 * ANTI_SPAMBOT variables, don't touch these if you don't understand what
 * is going on.
 * 
 * If a client joins MAX_JOIN_LEAVE_COUNT channels in a row, but spends
 * less than MIN_JOIN_LEAVE_TIME seconds on each one, flag it as a
 * possible spambot. disable JOIN for it and PRIVMSG but give no
 * indication to the client that this is happening. every time it tries
 * to JOIN OPER_SPAM_COUNTDOWN times, flag all opers on local server.
 * If a client doesn't LEAVE a channel for at least 2 minutes the
 * join/leave counter is decremented each time a LEAVE is done
 */
#define MIN_SPAM_NUM 5
#define MIN_SPAM_TIME 60
#define MIN_JOIN_LEAVE_TIME 60
#define MAX_JOIN_LEAVE_COUNT 25
#define OPER_SPAM_COUNTDOWN 5
#define JOIN_LEAVE_COUNT_EXPIRE_TIME 120

/*
 * If ANTI_SPAMBOT_WARN_ONLY is #define'd Warn opers about possible
 * spambots only, do not disable JOIN and PRIVMSG if possible spambot
 * is noticed Depends on your policies.
 */
#undef ANTI_SPAMBOT_WARN_ONLY

/*
 * If the OS has SOMAXCONN use that value, otherwise Use the value in
 * HYBRID_SOMAXCONN for the listen(); backlog try 5 or 25. 5 for AIX
 * and SUNOS, 25 should work better for other OS's
 */
#define HYBRID_SOMAXCONN 25

/* DO NOT EDIT ANYTHING BELOW THIS LINE! */

#ifdef INCREASE_SOCK_BUFS
#define CLIENT_BUFFER_SIZE 4096
#define SEND_BUFFER_SIZE 2048
#endif

#ifdef DEBUGMODE
extern void debug(int level, char *pattern, ...);
#define Debug(x) debug x
#define LOGFILE LOGFILE_DEFAULT
#else
#define Debug(x)
#define LOGFILE "/dev/null"
#endif

#define CONFIG_H_LEVEL_18

#endif /* __config_include__ */
