/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/dlink.h: double-linked list management
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: dlink.h,v 1.11 2004/03/22 23:17:13 rageircd Exp $
 */

#ifndef	__dlink_include__
#define	__dlink_include__

typedef struct _dlink_node dlink_node;
typedef struct _dlink_list dlink_list;
typedef struct _dlink_array dlink_array;

struct _dlink_node {
	dlink_node *next;
	dlink_node *prev;
	void *data;
};

struct _dlink_list {
	dlink_node *head;
	dlink_node *tail;
	unsigned long length;
};

struct _dlink_array {
	dlink_node nodes[MAXCONNECTIONS];
	int node_count;
};

#define DLINK_FOREACH(lst, ln) \
	for (ln = (lst); ln != NULL; ln = ln->next)

#define DLINK_FOREACH_PREV(lst, ln) \
	for (ln = (lst); ln != NULL; ln = ln->prev)

#define DLINK_FOREACH_DATA(lst, ln, d, dt) \
	for (ln = (lst), d = ln ? (dt *)ln->data : NULL; \
	  ln != NULL; \
	  ln = ln->next, d = ln ? (dt *)ln->data : NULL)

#define DLINK_FOREACH_PREV_DATA(lst, ln, d, dt) \
	for (ln = (lst), d = ln ? (dt *)ln->data : NULL; \
	  ln != NULL; \
	  ln = ln->prev, d = ln ? (dt *)ln->data : NULL)

#define DLINK_FOREACH_SAFE(lst, ln, nxt) \
	for (ln = (lst), nxt = ln ? ln->next : NULL; \
	  ln != NULL; \
	  ln = nxt, nxt = ln ? ln->next : NULL)

#define DLINK_FOREACH_PREV_SAFE(lst, ln, prv) \
	for (ln = (lst), prv = ln ? ln->prev : NULL; \
	  ln != NULL; \
	  ln = prv, prv = ln ? ln->next : NULL)

#define DLINK_FOREACH_SAFE_DATA(lst, ln, nxt, d, dt) \
	for (ln = (lst), nxt = ln ? ln->next : NULL, d = ln ? (dt *)ln->data : NULL; \
	  ln != NULL; \
	  ln = nxt, nxt = ln ? ln->next : NULL, d = ln ? (dt *)ln->data : NULL)

#define DLINK_FOREACH_PREV_SAFE_DATA(lst, ln, prv, d, dt) \
	for (ln = (lst), prv = ln ? ln->prev : NULL, d = ln ? (dt *)ln->data : NULL; \
	  ln != NULL; \
	  ln = prv, prv = ln ? ln->prev : NULL, d = ln ? (dt *)ln->data : NULL)

#define dlink_length(lst)	((lst)->length)
#define dlink_array_count(arr)	((arr)->node_count)

extern dlink_list lunknown_list;
extern dlink_list lclient_list;
extern dlink_list lserver_list;
extern dlink_list oper_list;
extern dlink_list listingcli_list;
extern dlink_list recvqcli_list;
extern dlink_list listener_list;
extern dlink_list connauth_list;

extern void dlink_init();

extern dlink_node *dlink_find(dlink_list *, void *);
extern dlink_node *dlink_add_node(dlink_list *, dlink_node *, void *);
extern dlink_node *dlink_add(dlink_list *, void *);
extern dlink_node *dlink_add_tail(dlink_list *, void *);
extern void dlink_del(dlink_list *, void *, dlink_node *);
extern dlink_node *dlink_del_nofree(dlink_list *, void *, dlink_node *);

extern void dlink_array_init(dlink_array *);
extern void dlink_array_add(dlink_array *, void *);

#endif /* __dlink_include__ */
