/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/fd.h: fd management header file
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: fd.h,v 1.17 2004/04/09 14:04:41 rageircd Exp $
 */

#ifndef __fd_include__
#define __fd_include__

#define FD_USED(x)	ASSERT(fd_table[x].open);
#define FD_UNUSED(x)	ASSERT(!fd_table[x].open);

enum {
	ENGINE_OK,
	ENGINE_ERR_BIND,
	ENGINE_ERR_DNS,
	ENGINE_ERR_TIMEOUT,
	ENGINE_ERR_CONNECT,
	ENGINE_ERROR,
	ENGINE_ERR_MAX
};

typedef struct _fd_stat fd_stat;
typedef enum _fd_list fd_list;
typedef struct _fd_entry fd_entry;

struct _fd_stat {
	int highest;
	int count;
};

enum _fd_list {
	FDL_NONE,
	FDL_ACTIVE,
	FDL_SERVER,
	FDL_SERVICE,
	FDL_CLIENT
};

struct _fd_entry {
	int fd;
	int engine_index;
	fd_list list;
	char desc[64];

	unsigned open : 1;
	unsigned short flags;

	void (*h_read)();
	void *d_read;

	void (*h_write)();
	void *d_write;

	time_t timeout;
	void (*h_timeout)();
	void *d_timeout;

	time_t flush_timeout;
	void (*h_flush)();
	void *d_flush;

	struct sockaddr_in ip;
	DNSQuery *dns_query;
	void (*h_connect)();
	void *d_connect;
};

extern fd_entry fd_table[MAXCONNECTIONS];
extern fd_stat fds;

extern void fd_init();
extern void fd_open(int, char *);
extern void fd_close(int);
#ifdef DEBUGMODE
extern void fd_report(aClient *);
#endif

#define FD_WANT_READ	0x1
#define FD_WANT_WRITE	0x2

#define fd_get_entry(xx)	&fd_table[xx]

extern void engine_write_error(aClient *, char *, int);
extern void engine_connect_error(aClient *, char *, int);
extern int ircsend(aClient *, const void *, size_t);
extern int ircrecv(aClient *, void *, size_t);
extern void engine_send_queued(int, void *);
extern void engine_read_packet(int, void *);
extern int engine_ignore_errno(int);
extern const char *engine_strerror(int);
extern int engine_open(int, int, char *);
extern int engine_accept(int fd, struct sockaddr_in *);
extern void engine_set_timeout(int fd, time_t, void (*)(), void *);
extern void engine_set_flush(int fd, time_t, void (*)(), void *);
extern void engine_check_timeouts();
extern void engine_connect_tcp(int, const char *, int, struct sockaddr *, int, void (*)(), void *, int);
extern void engine_preinit();
extern void engine_init();
extern void engine_set_call(int, fd_list, unsigned int, void (*)(), void *, time_t);
extern void engine_do_netio(time_t);

#ifdef USE_RTSIGIO
extern void setup_sigio_fd(int);
#endif

#endif /* __fd_include__ */
