/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/send.h: send header file
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: send.h,v 1.29 2004/04/10 23:39:59 rageircd Exp $
 */

#ifndef __send_include__
#define __send_include__

#include <stdarg.h>
#include "msg.h"

#define PREFIX_NONE	0
#define PREFIX_COLON	':'

extern void init_send();

extern void sendto_one_client(aClient *, aClient *, msg_ptr *, char *, ...);
extern void sendto_one_client_real(aClient *, aClient *, aClient *, msg_ptr *, char *, ...);
extern void sendto_one_client_prefixed(aClient *, aClient *, msg_ptr *, char *, ...);
extern void sendto_one_client_prefixed_copy(aClient *, aClient *, msg_ptr *, int, char *);
extern void sendto_one_client_nopostfix(aClient *, aClient *, msg_ptr *, char *, ...);
extern void sendto_one_client_nopostfix_copy(aClient *, aClient *, msg_ptr *, int, char *);
extern void sendto_one_client_onserver(aClient *, aClient *, msg_ptr *, char *, ...);
extern void sendto_one_client_onserver_copy(aClient *, aClient *, msg_ptr *, int, char *);

extern void sendto_client_numeric(aClient *, aClient *, char *, int, ...);
extern void sendto_serv_msg_butone(aClient *, aClient *, aClient *, msg_ptr *, char *, ...);
extern void sendto_serv_capab_msg_butone(aClient *, aClient *, aClient *, int, int, msg_ptr *, char *, ...);
extern void sendto_channel_msg_butone(aClient *, aClient *, aChannel *, char *, msg_ptr *, char *, ...);
extern void sendto_channel_msg_butone_copy(aClient *, aClient *, aChannel *, char *, msg_ptr *, int, char *);
extern void sendto_channel_msg_localremote_butone(int, aClient *, aClient *, aChannel *, char *, msg_ptr *, char *, ...);
extern void sendto_channel_msg_localremote_butone_copy(int, aClient *, aClient *, aChannel *, char *, msg_ptr *, int, char *);
extern void sendto_chanflag_msg_butone(aClient *, aClient *, aChannel *, char *, int, msg_ptr *, char *, ...);
extern void sendto_chanflag_msg_butone_copy(aClient *, aClient *, aChannel *, char *, int, msg_ptr *, int, char *);

#define send_me_notice(to, pat, args...)	sendto_one_client(to, &me, &CMD_NOTICE, pat, args)
#define send_me_noticeNA(to, pat)		sendto_one_client(to, &me, &CMD_NOTICE, pat)

#define send_me_numeric(to, num, args...)	sendto_client_numeric(to, &me, NULL, num, args)
#define send_me_numericNA(to, num)		sendto_client_numeric(to, &me, NULL, num)

#define send_me_numeric_buf(to, pat, num, args...) sendto_client_numeric(to, &me, pat, num, args)
#define send_me_numeric_bufNA(to, pat, num)	sendto_client_numeric(to, &me, pat, num)

#define send_me_debug(to, pat, args...)		sendto_client_numeric(to, &me, pat, RPL_STATSDEBUG, args)
#define send_me_debugNA(to, pat, args...)	sendto_client_numeric(to, &me, pat, RPL_STATSDEBUG)

#define send_channel_msg_local(from, chptr, msg, pattern, args...) \
	sendto_channel_msg_localremote_butone(1, NULL, from, chptr, chptr->chname, msg, pattern, args)
#define send_channel_msg_local_copy(from, chptr, msg, doprefix, buf) \
	sendto_channel_msg_localremote_butone_copy(1, NULL, from, chptr, chptr->chname, msg, doprefix, buf)

#define send_channel_msg_local_butone(one, from, chptr, msg, pattern, args...) \
	sendto_channel_msg_localremote_butone(1, one, from, chptr, chptr->chname, msg, pattern, args)
#define send_channel_msg_local_butone_copy(one, from, chptr, msg, dolocal, buf) \
	sendto_channel_msg_localremote_butone_copy(1, one, from, chptr, chptr->chname, msg, dolocal, buf)

extern void sendto_match_butone(aClient *, aClient *, char *, int, char *, ...);
extern void sendto_serv_butone(aClient *, char *, ...);
extern void sendto_common_channels(aClient *, char *, ...);
extern void sendto_wallops(aClient *, char *, ...);
extern void sendto_gnotice(char *, ...);
extern void sendto_one(aClient *, char *, ...);
extern void sendto_prefix_one(aClient *, aClient *, char *, ...);
extern void sendto_ops(char *, ...);
extern void sendto_realops(char *, ...);
extern void sendto_realops_lev(int, char *, ...);

extern void send_nick_to_serv_butone(aClient *, aClient *, char *);
extern void send_quit_to_common_channels(aClient *, char *);
extern void send_part_to_common_channels(aClient *, char *);
extern void send_chatops(char *, ...);
extern void send_globops(char *, ...);
extern void send_locops(char *, ...);

extern void ts_warn(char *, ...);
extern void flush_queued_buffers();

#endif /* __send_include__ */
