/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_admin.c: ADMIN command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_admin.c,v 1.28 2004/03/07 17:11:09 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "hook.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_admin) = {
	"modules/commands/m_admin.so",
	"/ADMIN command",
	5, "$Revision: 1.28 $"
};

int MOD_LOAD(m_admin)()
{
	register_command(&MOD_HEADER(m_admin), &CMD_ADMIN, m_admin);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_admin)()
{
	return MOD_SUCCESS;
}

/*
 * m_admin
 *	parv[0] = sender prefix
 *	parv[1] = servername
 */
int m_admin(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	static time_t last_used = 0L;

	if (use_or_deliver(cptr, sptr, &CMD_ADMIN, ":%s", 1, parc, parv) != HUNTED_ISME) {
		return 0;
	}
	if (!HasMode(sptr, UMODE_OPER)) {
		if (FloodConfig.pace_wait_simple && (last_used + FloodConfig.pace_wait_simple > timeofday)) {
			send_me_numericNA(sptr, RPL_LOAD2HI);
			return 0;
		}
		else {
			last_used = timeofday;
		}
		if (GeneralConfig.spy_notices && IsPerson(sptr)) {
			sendto_realops_lev(SPY_LEV, "ADMIN requested by %s (%s@%s) [%s]", sptr->name,
				sptr->username, MaskedHost(sptr), sptr->user->server);
		}
	}
	if (AdminInfo == NULL) {
		send_me_numeric(sptr, ERR_NOADMININFO, me.name);
		return 0;
	}

	send_me_numeric(sptr, RPL_ADMINME, me.name);
	if (!BadPtr(AdminInfo->name)) {
		send_me_numeric(sptr, RPL_ADMINLOC1, AdminInfo->name);
	}
	if (!BadPtr(AdminInfo->desc)) {
		send_me_numeric(sptr, RPL_ADMINLOC2, AdminInfo->desc);
	}
	if (!BadPtr(AdminInfo->email)) {
		send_me_numeric(sptr, RPL_ADMINEMAIL, AdminInfo->email);
	}
	return 0;
}
