/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_die.c: DIE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_die.c,v 1.30 2004/03/07 16:58:29 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_die) = {
	"modules/commands/m_die.so",
	"/DIE command",
	5, "$Revision: 1.30 $"
};

int MOD_LOAD(m_die)()
{
	register_command(&MOD_HEADER(m_die), &CMD_DIE, m_die);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_die)()
{
	return MOD_SUCCESS;
}

/*
 * m_die
 *	parv[0] = sender prefix
 *	parv[1] = password
 */
int m_die(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	dlink_node *node;
	aClient *acptr = NULL;
	char *inpath;

	if (!OPHasFlag(sptr, OFLAG_DIE)) {
		send_me_numericNA(sptr, ERR_NOPRIVILEGES);
		return 0;
	}

	DLINK_FOREACH_DATA(lclient_list.head, node, acptr, aClient) {
		send_me_notice(acptr, ":Server terminating by request of %s (%s@%s).",
			sptr->name, sptr->username, MaskedHost(sptr));
	}

	inpath = get_client_name(sptr, HIDE_IP);
	DLINK_FOREACH_DATA(lserver_list.head, node, acptr, aClient) {
		sendto_one_client_nopostfix(acptr, NULL, &CMD_ERROR, ":Terminated by %s",
			inpath);
	}

	ircdlog(LOG_DEFAULT, "Server terminating by request of %s", sptr->name, inpath);
	server_die();

	return 0;
}
