/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_error.c: ERROR protocol module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_error.c,v 1.14 2004/03/07 16:58:29 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_error) = {
	"modules/commands/m_error.so",
	"ERROR protocol",
	5, "$Revision: 1.14 $"
};

int MOD_LOAD(m_error)()
{
	register_command(&MOD_HEADER(m_error), &CMD_ERROR, m_error);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_error)()
{
	return MOD_SUCCESS;
}

/*
 * m_error
 *	parv[0] = sender prefix
 *	parv[1] = message
 */
int m_error(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *para = NULL, *path = get_client_name(cptr, HIDE_IP);

	if (IsPerson(cptr) || IsUnknown(cptr)) {
		return 0;
	}

	para = (parc > 1 && *parv[1] != '\0') ? parv[1] : "<>";
	Debug((DEBUG_ERROR, "Received ERROR message from %s: %s", sptr->name, para));

	if (cptr == sptr) {
		sendto_gnotice("ERROR from %s: %s", path, para);
		sendto_serv_msg_butone(cptr, &me, NULL, &CMD_GNOTICE, ":ERROR from %s: %s",
			path, para);
	}
	else {
		sendto_gnotice("ERROR from %s via %s: %s", sptr->name, path, para);
		sendto_serv_msg_butone(cptr, &me, NULL, &CMD_GNOTICE, ":ERROR from %s via %s: %s",
			sptr->name, path, para);
	}
	return 0;
}
