/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_globops.c: GLOBOPS command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_globops.c,v 1.16 2004/03/07 16:58:29 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_globops) = {
	"modules/commands/m_globops.so",
	"/GLOBOPS command",
	5, "$Revision: 1.16 $"
};

int MOD_LOAD(m_globops)()
{
	register_command(&MOD_HEADER(m_globops), &CMD_GLOBOPS, m_globops);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_globops)()
{
	return MOD_SUCCESS;
}

/*
 * m_globops
 *	parv[0] = sender prefix
 *	parv[1] = message
 */
int m_globops(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	if (MyClient(sptr) && !OPHasFlag(sptr, OFLAG_GLOBOPS) && !IsULine(sptr)) {
		send_me_numericNA(sptr, ERR_NOPRIVILEGES);
		return 0;
	}
	if (parc < 2 || *parv[1] == '\0') {
		if (MyClient(sptr)) {
			send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "GLOBOPS");
		}
		return 0;
	}
	if (strlen(parv[1]) > TOPICLEN) {
		parv[1][TOPICLEN] = '\0';
	}

	send_globops("from %s: %s", parv[0], parv[1]);
	sendto_serv_msg_butone(cptr, sptr, NULL, &CMD_GLOBOPS, ":%s", parv[1]);
	return 0;
}
