/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_invite.c: INVITE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_invite.c,v 1.21 2004/03/07 16:58:29 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_invite) = {
	"modules/commands/m_invite.so",
	"/INVITE command",
	5, "$Revision: 1.21 $"
};

int MOD_LOAD(m_invite)()
{
	register_command(&MOD_HEADER(m_invite), &CMD_INVITE, m_invite);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_invite)()
{
	return MOD_SUCCESS;
}

/*
 * m_invite
 *	parv[0] = sender prefix
 *	parv[1] = user to invite
 *	parv[2] = channel
 */
int m_invite(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr;
	aChannel *chptr;

	if (parc < 3 || *parv[1] == '\0') {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "INVITE");
		return 0;
	}

	if ((acptr = find_person(parv[1], NULL)) == NULL) {
		target_left(sptr, parv[1], MSG_INVITE, parv[2]);
		return 0;
	}
	if (!check_channelname(sptr, (unsigned char *)parv[2])) {
		return 0;
	}
	if ((chptr = find_channel(parv[2], NULL)) == NULL) {
		if (MyClient(sptr)) {
			send_me_numeric(sptr, ERR_NOSUCHCHANNEL, parv[2]);
		}
		else {
			sendto_one_client_prefixed_copy(acptr, sptr, &CMD_INVITE, PREFIX_NONE,
				parv[2]);
		}
		return 0;
	}
	if (!IsMember(sptr, chptr) && !IsULine(sptr)) {
		send_me_numeric(sptr, ERR_NOTONCHANNEL, parv[2]);
		return 0;
	}
	if (IsMember(acptr, chptr)) {
		send_me_numeric(sptr, ERR_USERONCHANNEL, parv[1], parv[2]);
		return 0;
	}
	if (chptr->mode.mode & CMODE_INVITEONLY) {
		if (!is_chanop(sptr, chptr) && !is_halfop(sptr, chptr) && !IsULine(sptr)) {
			send_me_numeric(sptr, ERR_CHANOPRIVSNEEDED, chptr->chname);
			return 0;
		}
		else if (!IsMember(sptr, chptr) && !IsULine(sptr)) {
			send_me_numeric(sptr, ERR_CHANOPRIVSNEEDED,
				(chptr != NULL) ? chptr->chname : parv[2]);
			return 0;
		}
	}
	if (MyConnect(sptr)) {
		send_me_numeric(sptr, RPL_INVITING, acptr->name,
			(chptr != NULL) ? chptr->chname : parv[2]);
		if (acptr->user->away != NULL) {
			send_me_numeric(sptr, RPL_AWAY, acptr->name, acptr->user->away);
		}
	}
	if (MyConnect(acptr)) {
		if ((sptr->user != NULL && is_chanop(sptr, chptr)) || IsULine(sptr)) {
			char pfixed_chname[CHANNELLEN + 2];

			add_invite(acptr, chptr);
			ircsprintf(pfixed_chname, "@%s", chptr->chname);
			sendto_chanflag_msg_butone(NULL, &me, chptr, pfixed_chname, CMODE_CHANOP,
				&CMD_NOTICE, "%s invited %s into channel %s.", sptr->name,
				acptr->name, chptr->chname);
		}
	}
	sendto_one_client_prefixed_copy(acptr, sptr, &CMD_INVITE, PREFIX_NONE, chptr->chname);
	return 0;
}
