/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_ison.c: ISON command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_ison.c,v 1.17 2004/03/07 16:58:29 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>  
#include <string.h>
#include <fcntl.h>

static char buf[BUFSIZE];

Module MOD_HEADER(m_ison) = {
	"modules/commands/m_ison.so",
	"/ISON command",
	5, "$Revision: 1.17 $"
};

int MOD_LOAD(m_ison)()
{
	register_command(&MOD_HEADER(m_ison), &CMD_ISON, m_ison);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_ison)()
{
	return MOD_SUCCESS;
}

/*
 * m_ison
 *	parv[0] = sender prefix
 *	parv[1] = nickname list
 */
int m_ison(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr;
	char *s, **pav = parv, *p = NULL, *t;
	int len = 0, mlen;

	if (parc < 2 || *parv[1] == '\0') {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "ISON");
		return 0;
	}

	mlen = strlen(me.name) + strlen(sptr->name) + 12;
	for (s = strtoken(&p, *++pav, " "); s != NULL; s = strtoken(&p, NULL, " ")) {
		if ((acptr = find_person(s, NULL)) != NULL) {
			if ((strlen(acptr->name) + mlen + 1) < sizeof(buf)) {
				if (len > 0) {
					buf[len++] = ' ';
				}
				for (t = acptr->name; *t != '\0'; t++) {
					buf[len++] = *t;
				}
			}
			else {
				break;
			}
		}
	}
	buf[len] = '\0';

	send_me_numeric(sptr, RPL_ISON, buf);
	return 0;
}
