/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_kick.c: KICK command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_kick.c,v 1.30 2004/05/12 19:48:22 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "send.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_kick) = {
	"modules/commands/m_kick.so",
	"/KICK command",
	6, "$Revision: 1.30 $"
};

int MOD_LOAD(m_kick)()
{
	register_command(&MOD_HEADER(m_kick), &CMD_KICK, m_kick);
	MOD_SET_FLAG(&MOD_HEADER(m_kick), MOD_FLAG_PERM);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_kick)()
{
	return MOD_SUCCESS;
}

/*
 * m_kick
 *	parv[0] = sender prefix
 *	parv[1] = channel list
 *	parv[2] = client list
 *	parv[3] = comment
 */
int m_kick(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr = NULL;
	aChannel *chptr = NULL;
	char *name, *user, *p = NULL, *p2 = NULL;
	char *comment = (parc > 3 && *parv[3] != '\0') ? parv[3] : parv[0];

	if (parc < 3 || *parv[1] == '\0') {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "KICK");
		return 0;
	}
	if (strlen(comment) > TOPICLEN) {
		comment[TOPICLEN] = '\0';
	}
	for (; (name = strtoken(&p, parv[1], ",")); parv[1] = NULL) {
		if ((chptr = find_channel(name, NULL)) == NULL) {
			send_me_numeric(sptr, ERR_NOSUCHCHANNEL, name);
			continue;
		}
		if (!IsServer(sptr) && !is_chanop(sptr, chptr) && !is_halfop(sptr, chptr) && !IsULine(sptr)) {
			if (MyConnect(sptr) || !chptr->channelts) {
				send_me_numeric(sptr, ERR_CHANOPRIVSNEEDED, chptr->chname);
				continue;
			}
		}

		for (; (user = strtoken(&p2, parv[2], ",")); parv[2] = NULL) {
			if ((acptr = IsServer(cptr) ? find_target_chasing(sptr, user, MSG_KICK) :
			  find_chasing(sptr, user, NULL, MSG_KICK)) == NULL) {
				continue;
			}
			if (!IsMember(acptr, chptr)) {
				send_me_numeric(sptr, ERR_USERNOTINCHANNEL, user, name);
				continue;
			}
			if (!IsServer(sptr) && !IsULine(sptr)) {
				if (!is_chanadmin(sptr, chptr) && is_chanadmin(acptr, chptr)) {
					send_me_notice(sptr, ":Cannot kick channel administrator "
						"%s", user);
					continue;
				}
				if (is_halfop(sptr, chptr) && !is_chanop(sptr, chptr) &&
				  is_chanop(acptr, chptr)) {
					send_me_notice(sptr, ":Cannot kick channel operator %s",
						user);
					continue;
				}
			}

			send_channel_msg_local(sptr, chptr, &CMD_KICK, "%s :%s", acptr->name,
				comment);

			sendto_serv_capab_msg_butone(cptr, sptr, NULL, 0, CAP_SUID|CAP_TOK1,
				&CMD_KICK, "%s %s :%s", chptr->chname, acptr->name, comment);
			if (HasSUID(acptr)) {
				sendto_serv_capab_msg_butone(cptr, sptr, NULL, CAP_SUID|CAP_TOK1, 0,
					&CMD_KICK, "%s !%s :%s", chptr->chname, acptr->id.string, comment);
			}
			else {
				sendto_serv_capab_msg_butone(cptr, sptr, NULL, CAP_SUID|CAP_TOK1, 0,
					&CMD_KICK, "%s %s :%s", chptr->chname, acptr->name, comment);
			}

			remove_user_from_channel(acptr, chptr);
		}
	}
	return 0;
}
