/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_knock.c: KNOCK command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_knock.c,v 1.26 2004/03/07 17:11:09 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_knock) = {
	"modules/commands/m_knock.so",
	"/KNOCK command",
	5, "$Revision: 1.26 $"
};

int MOD_LOAD(m_knock)()
{
	register_command(&MOD_HEADER(m_knock), &CMD_KNOCK, m_knock);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_knock)()
{
	return MOD_SUCCESS;
}

/*
 * m_knock
 *	parv[0] = sender prefix
 *	parv[1] = channel name
 *	parv[2] = optional channel key
 */
int m_knock(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aChannel *chptr;
	char *p, *name, *key, pfixed_chname[CHANNELLEN + 3];

	if (!GeneralConfig.enable_knock) {
		send_me_numeric(sptr, ERR_FUNCDISABLED, "KNOCK");
		return 0;
	}
	if (parc < 2 || BadPtr(parv[1])) {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "KNOCK");
		return 0;
	}
	if (!check_channelname(sptr, parv[1])) {
		return 0;
	}

	name = parv[1];
	key = (parc > 2 && *parv[2] != '\0') ? parv[2] : NULL;
	if ((p = strchr(name, ',')) != NULL) {
		*p = '\0';
	}
	if (!IsChannelName(name) || (chptr = find_channel(name, NULL)) == NULL) {
		send_me_numeric(sptr, ERR_NOSUCHCHANNEL, name);
		return 0;
	}
	if (IsMember(sptr, chptr)) {
		send_me_numeric(sptr, RPL_KNOCKONCHAN, chptr->chname);
		return 0;
	}
	if (!(chptr->mode.mode & CMODE_INVITEONLY) && ((!chptr->mode.limit || (chptr->mode.limit
	  && chptr->users < chptr->mode.limit)) || ((chptr->mode.mode & CMODE_INVITEONLY) &&
	  has_invex(sptr, chptr)))) {
		send_me_numeric(sptr, RPL_KNOCKCHANOPEN, chptr->chname);
		return 0;
	}
	if (has_ban(sptr, chptr)) {
		send_me_numeric(sptr, ERR_BANNEDFROMCHAN, chptr->chname);
		return 0;
	}
	if (*chptr->mode.key != '\0' && (key == NULL || irccmp(chptr->mode.key, key))) {
		send_me_numeric(sptr, ERR_BADCHANNELKEY, chptr->chname);
		return 0;
	}
	if (chptr->mode.mode & CMODE_PRIVATE) {
		send_me_numeric(sptr, ERR_CANNOTSENDTOCHAN, chptr->chname, "channel is private");
		return 0;
	}
	if (FloodConfig.knock_delay && ((chptr->last_knock + FloodConfig.knock_delay) > timeofday)) {
		send_me_numeric(sptr, RPL_KNOCKWAIT, chptr->chname,
			FloodConfig.knock_delay - (timeofday - chptr->last_knock));
		return 0;
	}

	chptr->last_knock = timeofday;
	ircsprintf(pfixed_chname, "@%%%s", chptr->chname);
	sendto_chanflag_msg_butone(NULL, &me, chptr, pfixed_chname, CMODE_CHANOP|CMODE_HALFOP,
		&CMD_NOTICE, "%s (%s@%s) has requested an invite into %s", parv[0], sptr->username,
		MaskedHost(sptr), chptr->chname);
	send_me_numeric(sptr, RPL_KNOCKDELIVERED, chptr->chname);
	return 0;
}
