/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_locops.c: LOCOPS command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_locops.c,v 1.14 2004/03/07 16:58:29 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_locops) = {
	"modules/commands/m_locops.so",
	"/LOCOPS command",
	5, "$Revision: 1.14 $"
};

int MOD_LOAD(m_locops)()
{
	register_command(&MOD_HEADER(m_locops), &CMD_LOCOPS, m_locops);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_locops)()
{
	return MOD_SUCCESS;
}

/*
 * m_locops
 *	parv[0] = sender prefix
 *	parv[1] = message
 */
int m_locops(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	if (!IsServer(sptr) && MyConnect(sptr) && !OPHasFlag(sptr, OFLAG_LOCOPS)) {
		send_me_numericNA(sptr, ERR_NOPRIVILEGES);
		return 0;
	}
	if (parc < 2 || *parv[1] == '\0') {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "LOCOPS");
		return 0;
	}
	send_locops("from %s: %s", parv[0], parv[1]);
	return 0;
}
