/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_myid.c: MYID protocol module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_myid.c,v 1.6 2004/03/07 16:58:30 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_myid) = {
	"modules/commands/m_myid.so",
	"/MYID protocol",
	5, "$Revision: 1.6 $"
};

int MOD_LOAD(m_myid)()
{
	register_command(&MOD_HEADER(m_myid), &CMD_MYID, m_myid);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_myid)()
{
	return MOD_SUCCESS;
}

/*
 * m_myid
 *	parv[0] = base64 server id
 */
int m_myid(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *b64id;

	if (!(IsUnknown(cptr) || IsServer(cptr) || IsHandshake(cptr) || IsConnecting(cptr)) ||
	  !BadPtr(cptr->id.string) || parc < 2 || !CapID(cptr) || cptr != sptr) {
		return 0;
	}

	b64id = parv[1];
	if (*b64id != '!' || b64id[1] == '\0' || !valid_base64_servid(b64id + 1)) {
		return 0;
	}

	b64id++;
	strncpy(cptr->id.string, b64id, 8);
	return 0;
}
