/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_names.c: NAMES command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_names.c,v 1.27 2004/05/13 16:12:54 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "hook.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

static int send_channel_names(HookData *);

Module MOD_HEADER(m_names) = {
	"modules/commands/m_names.so",
	"/NAMES command",
	6, "$Revision: 1.27 $"
};

int MOD_LOAD(m_names)()
{
	Hook *hook;

	if ((hook = hook_find_from_module("h_post_join", "modules/commands/m_join.so")) == NULL) {
		ircdlog(LOG_ERROR, "m_names.so: couldn't find hook h_post_join");
		return MOD_FAILURE;
	}

	register_hook_event(&MOD_HEADER(m_names), hook, send_channel_names);
	register_command(&MOD_HEADER(m_names), &CMD_NAMES, m_names);

	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_names)()
{
	return MOD_SUCCESS;
}

static void send_names(aClient *cptr, aChannel *chptr)
{
	int flag = 1, len = 0, mlen = (strlen(me.name) + NICKLEN + 7);
	int prefix_len, member = (IsMember(cptr, chptr) ? 1 : 0);
	chanMember *cm;
	aClient *acptr;
	char *s, buf[BUFSIZE];

	if (PubChannel(chptr)) {
		buf[0] = '=';
	}
	else if (SecretChannel(chptr)) {
		buf[0] = '@';
	}
	else {
		buf[0] = '*';
	}

	len = 1;
	buf[len++] = ' ';
	for (s = chptr->chname; *s != '\0'; s++) {
		buf[len++] = *s;
	}
	buf[len++] = ' ';
	buf[len++] = ':';
	buf[len] = '\0';
	prefix_len = len;

	for (cm = chptr->members; cm != NULL; cm = cm->nextuser) {
		acptr = cm->cptr;
		if (HasMode(acptr, UMODE_INVISIBLE) && !member) {
			continue;
		}
		if (cm->flags & CMODE_CHANOP) {
			buf[len++] = '@';
		}
		else if (cm->flags & CMODE_HALFOP) {
			buf[len++] = '%';
		}
		else if (cm->flags & CMODE_VOICE) {
			buf[len++] = '+';
		}
		for (s = acptr->name; *s != '\0'; s++) {
			buf[len++] = *s;
		}
		buf[len++] = ' ';
		buf[len] = '\0';
		flag = 1;

		if (mlen + len + NICKLEN > BUFSIZE - 3) {
			send_me_numeric(cptr, RPL_NAMREPLY, buf);
			len = prefix_len;
			flag = 0;
		}
	}
	if (flag) {
		send_me_numeric(cptr, RPL_NAMREPLY, buf);
	}
	send_me_numeric(cptr, RPL_ENDOFNAMES, chptr->chname);
	return;
}

static int send_channel_names(HookData *hdata)
{
	send_names(hdata->sptr, hdata->chptr);
	return 0;
}

/*
 * m_names
 *	parv[0] = sender prefix
 *	parv[1] = channel
 */
int m_names(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aChannel *chptr;
	char *s, *para = parc > 1 ? parv[1] : NULL;

	if (parc < 2 || !MyConnect(sptr)) {
		send_me_numeric(sptr, RPL_ENDOFNAMES, "*");
		return 0;
	}
	for (s = para; *s != '\0'; s++) {
		if (*s == ',') {
			if (strlen(para) > 64) {
				para[64] = '\0';
			}
			sendto_realops_lev(FLOOD_LEV, "Possible names abuser: %s (%s@%s) target(s): %s",
				sptr->name, sptr->username, MaskedHost(sptr), para);
			send_me_numeric(sptr, ERR_TOOMANYTARGETS, "NAMES");
			return 0;
		}
	}
	if (!check_channelname(sptr, (unsigned char *)para)) {
		return 0;
	}
	if ((chptr = find_channel(para, NULL)) == NULL || !ShowChannel(sptr, chptr)) {
		send_me_numeric(sptr, RPL_ENDOFNAMES, para);
		return 0;
	}
	send_names(sptr, chptr);
	return 0;
}
