/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_opermotd.c: OPERMOTD command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_opermotd.c,v 1.19 2004/04/07 15:08:22 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_opermotd) = {
	"modules/commands/m_opermotd.so",
	"/OPERMOTD command",
	5, "$Revision: 1.19 $"
};

int MOD_LOAD(m_opermotd)()
{
	register_command(&MOD_HEADER(m_opermotd), &CMD_OPERMOTD, m_opermotd);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_opermotd)()
{
	return MOD_SUCCESS;
}

/*
 * m_opermotd
 *	parv[0] = sender prefix
 *	parv[1] = server
 */
int m_opermotd(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	if (use_or_deliver(cptr, sptr, &CMD_OPERMOTD, ":%s", 1, parc, parv) != HUNTED_ISME) {
		return 0;
	}
	if (!HasMode(sptr, UMODE_OPER)) {
		if (MyClient(sptr)) {
			send_me_numericNA(sptr, ERR_NOPRIVILEGES);
		}
		return 0;
	}

	send_message_file(sptr, opermotd);
	return 0;
}
