/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_part.c: PART command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_part.c,v 1.32 2004/05/08 16:12:02 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_part) = {
	"modules/commands/m_part.so",
	"/PART command",
	6, "$Revision: 1.32 $"
};

int MOD_LOAD(m_part)()
{
	register_command(&MOD_HEADER(m_part), &CMD_PART, m_part);
	MOD_SET_FLAG(&MOD_HEADER(m_part), MOD_FLAG_PERM);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_part)()
{
	return MOD_SUCCESS;
}

/*
 * m_part
 *	parv[0] = sender prefix
 *	parv[1] = channel name
 *	parv[2] = message
 */
int m_part(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aChannel *chptr;
	char *p, *name, *reason = (parc > 2 && parv[2] && *parv[2] != '\0') ? parv[2] : NULL;
	int ret = 0;

	if (parc < 2 || *parv[1] == '\0') {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "PART");
		return 0;
	}

	name = strtoken(&p, parv[1], ",");
#ifdef ANTI_SPAMBOT
	if (name != NULL && MyConnect(sptr) && !HasMode(sptr, UMODE_OPER)) {
		if (sptr->localUser->join_part_count >= spam_num) {
			sendto_realops_lev(SPAM_LEV, "User %s (%s@%s) is a possible spambot", sptr->name,
				sptr->username, MaskedHost(sptr));
			sptr->localUser->oper_warn_countdown = OPER_SPAM_COUNTDOWN;
		}
		else {
			int t_delta = (timeofday - sptr->localUser->last_part_time);
			if (t_delta > JOIN_LEAVE_COUNT_EXPIRE_TIME) {
				int decrement_count = (t_delta / JOIN_LEAVE_COUNT_EXPIRE_TIME);
				if (decrement_count > sptr->localUser->join_part_count) {
					sptr->localUser->join_part_count = 0;
				}
				else {
					sptr->localUser->join_part_count -= decrement_count;
				}
			}
			else if ((timeofday - sptr->localUser->last_join_time) < spam_time) {
				sptr->localUser->join_part_count++;
			}
			sptr->localUser->last_part_time = timeofday;
		}
	}
#endif
	while (name != NULL) {
		if ((chptr = get_channel(sptr, name, 0, NULL)) == NULL) {
			send_me_numeric(sptr, ERR_NOSUCHCHANNEL, name);
			name = strtoken(&p, NULL, ",");
			continue;
		}
		if (!IsMember(sptr, chptr)) {
			send_me_numeric(sptr, ERR_NOTONCHANNEL, name);
			name = strtoken(&p, NULL, ",");
			continue;
		}

		ret = IsULine(sptr) ? 0 : can_send(sptr, chptr, reason);
		if (ret && MyClient(sptr)) {
			if (ret == CMODE_NOCOLOUR) {
				send_me_numeric(sptr, ERR_NOCOLOURSONCHAN, name, reason);
			}
		}
		if (parc < 3 || ret > 0) {
			sendto_serv_msg_butone(cptr, sptr, NULL, &CMD_PART, "%s", name);
			send_channel_msg_local_copy(sptr, chptr, &CMD_PART, PREFIX_NONE, "");
		}
		else {
			sendto_serv_msg_butone(cptr, sptr, NULL, &CMD_PART, "%s :%s", name, reason);
			send_channel_msg_local_copy(sptr, chptr, &CMD_PART, PREFIX_COLON, reason);
		}
		remove_user_from_channel(sptr, chptr);
		name = strtoken(&p, NULL, ",");
	}
	return 0;
}
