/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_pong.c: PONG command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_pong.c,v 1.38 2004/05/08 16:12:02 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_pong) = {
	"modules/commands/m_pong.so",
	"/PONG command",
	6, "$Revision: 1.38 $"
};

int MOD_LOAD(m_pong)()
{
	register_command(&MOD_HEADER(m_pong), &CMD_PONG, m_pong);
	MOD_SET_FLAG(&MOD_HEADER(m_pong), MOD_FLAG_PERM);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_pong)()
{
	return MOD_SUCCESS;
}

/*
 * m_pong
 *	parv[0] = sender prefix
 *	parv[1] = origin
 *	parv[2] = destination
 */
int m_pong(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr = NULL;
	char *origin, *destination;

	if (parc < 2 || *parv[1] == '\0') {
		send_me_numericNA(sptr, ERR_NOORIGIN);
		return 0;
	}

	origin = parv[1];
	destination = parv[2];

	ClearPingSent(cptr);
	ClearPingSent(sptr);

	if ((sptr == cptr) && IsServer(sptr)) {
		ASSERT(sptr->localClient != NULL);
		if (UserBurst(sptr)) {
			sendto_gnotice("%s has processed user/channel burst, sending topic burst.",
				sptr->name);

			ClearUserBurst(sptr);
			send_topic_burst(sptr);

			SetPingSent(sptr);
			SetSentSOB(sptr);

			sendto_one_client_nopostfix(sptr, &me, &CMD_PING, ":%s", me.name);
		}
		else if (TopicBurst(sptr)) {
			char *cap_list = get_synch_cap(sptr->localClient->capabs);

			ClearTopicBurst(sptr);

			sendto_gnotice("%s has processed topic burst, synch complete. (states: %s)",
				sptr->name, !BadPtr(cap_list) ? cap_list : "none");
			if (ServerInfo->hub) {
				sendto_serv_msg_butone(sptr, &me, NULL, &CMD_GNOTICE, ":%s has processed topic "
					"burst, synch complete. (states: %s)", sptr->name,
					!BadPtr(cap_list) ? cap_list : "none");
			}
			sendto_one_client_nopostfix(sptr, &me, &CMD_PING, ":%s", me.name);
		}
	}
	if (!BadPtr(destination) && mycmp(destination, me.name) && IsRegistered(sptr)) {
		if ((acptr = find_client(destination, NULL)) == NULL ||
		  (acptr = find_server(destination, NULL)) == NULL) {
			send_me_numeric(sptr, ERR_NOSUCHSERVER, destination);
			return 0;
		}
		sendto_one_client_nopostfix(acptr, sptr, &CMD_PONG, "%s %s", origin,
			destination);
	}
#ifdef DEBUGMODE
	else {
		Debug((DEBUG_NOTICE, "PONG: %s %s", origin, destination ? destination : "*"));
	}
#endif
	return 0;
}
