/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_quit.c: QUIT command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_quit.c,v 1.26 2004/05/13 22:11:45 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_quit) = {
	"modules/commands/m_quit.so",
	"/QUIT command",
	6, "$Revision: 1.26 $"
};

int MOD_LOAD(m_quit)()
{
	register_command(&MOD_HEADER(m_quit), &CMD_QUIT, m_quit);
	MOD_SET_FLAG(&MOD_HEADER(m_quit), MOD_FLAG_PERM);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_quit)()
{
	return MOD_SUCCESS;
}

/*
 * m_quit
 *	parv[0] = sender prefix
 *	parv[1] = comment
 */
int m_quit(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	int validmsg = (parc > 1 && *parv[1] != '\0') ? 1 : 0;
	char comment[TOPICLEN + 1];

	SetNormalExit(sptr);

	if (IsServer(cptr)) {
		return exit_client(cptr, sptr, sptr, validmsg ? parv[1] : cptr->name);
	}

	if (!validmsg || !GeneralConfig.custom_quit_msgs || (!HasMode(sptr, UMODE_OPER)
	  && ((FloodConfig.antispam_quit_msg_time > 0)
	  && (sptr->firsttime + FloodConfig.antispam_quit_msg_time) > timeofday))) {
		return exit_client(cptr, sptr, sptr, "Client Quit");
	}

	if (strlen(parv[1]) > (TOPICLEN - 7)) {
		parv[1][TOPICLEN - 7] = '\0';
	}
	ircsprintf(comment, "Quit: %s", parv[1]);

	return exit_client(cptr, sptr, sptr, comment);
}
