/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_rakill.c: RAKILL command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_rakill.c,v 1.18 2004/03/07 16:58:30 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

extern void send_globops(char *, ...);

Module MOD_HEADER(m_rakill) = {
	"modules/commands/m_rakill.so",
	"/RAKILL command",
	5, "$Revision: 1.18 $"
};

int MOD_LOAD(m_rakill)()
{
	register_command(&MOD_HEADER(m_rakill), &CMD_RAKILL, m_rakill);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_rakill)()
{
	return MOD_SUCCESS;
}

/*
 * m_rakill
 *	parv[0] = sender prefix
 *	parv[1] = host
 *	parv[2] = user
 */
int m_rakill(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *userhost;
	dlink_node *node, *next = NULL;
	ConfigItem_ban *ban;

	if (!IsServer(sptr) || parc < 3) {
		return 0;
	}
	if (!IsULine(sptr)) {
		send_globops("Non-ULined server %s is trying to RAKILL!", parv[0]);
		sendto_serv_msg_butone(NULL, &me, NULL, &CMD_GLOBOPS, ":Non-ULined server %s is trying "
			"to RAKILL!", parv[0]);
		return 0;
	}

	userhost = make_user_host(parv[2], parv[1]);
	DLINK_FOREACH_SAFE_DATA(conf_ban_list.head, node, next, ban, ConfigItem_ban) {
		if ((ban->flags & (BAN_USERHOST|BAN_SVS)) != (BAN_USERHOST|BAN_SVS)) {
			continue;
		}
		if (!match(ban->mask, userhost)) {
			dlink_del(&conf_ban_list, NULL, node);
			free_ban(ban);
			break;
		}
	}
	sendto_serv_msg_butone(cptr, sptr, NULL, &CMD_RAKILL, "%s %s", parv[1], parv[2]);
	return 0;
}
