/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_rehash.c: REHASH command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_rehash.c,v 1.35 2004/04/09 23:11:33 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include "res.h"
#include "blalloc.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_rehash) = {
	"modules/commands/m_rehash.so",
	"/REHASH command",
	5, "$Revision: 1.35 $"
};

int MOD_LOAD(m_rehash)()
{
	register_command(&MOD_HEADER(m_rehash), &CMD_REHASH, m_rehash);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_rehash)()
{
	return MOD_SUCCESS;
}

/*
 * m_rehash
 *	parv[0] = sender prefix
 *	parv[1] = options
 */
int m_rehash(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	int all = 0;

	if (!OPHasFlag(sptr, OFLAG_REHASH)) {
		send_me_numericNA(sptr, ERR_NOPRIVILEGES);
		return 0;
	}
	if (parc > 1) {
		if (!mycmp(parv[1], "-motd")) {
			send_me_numeric(sptr, RPL_REHASHING, "MOTD");
			sendto_ops("%s is rehashing MOTD", parv[0]);

			if (motd != NULL) {
				destroy_message_file(motd);
			}
			motd = load_message_file(MOTD_FILE, MFTYPE_MOTD);
		}
		else if (!mycmp(parv[1], "-opermotd")) {
			send_me_numeric(sptr, RPL_REHASHING, "OPERMOTD");
			sendto_ops("%s is rehashing OPERMOTD", parv[0]);

			if (opermotd != NULL) {
				destroy_message_file(opermotd);
			}
			opermotd = load_message_file(OPERMOTD_FILE, MFTYPE_OPERMOTD);
		}
		else if (!mycmp(parv[1], "-rules")) {
			send_me_numeric(sptr, RPL_REHASHING, "RULES");
			sendto_ops("%s is rehashing RULES", parv[0]);

			if (rules != NULL) {
				destroy_message_file(rules);
			}
			rules = load_message_file(RULES_FILE, MFTYPE_RULES);
		}
#ifdef USE_THROTTLE
		else if (!mycmp(parv[1], "-throttle")) {
			send_me_numeric(sptr, RPL_REHASHING, "THROTTLE");
			sendto_ops("%s is rehashing THROTTLE", parv[0]);
			throttle_rehash();
		}
#endif
		else if (!mycmp(parv[1], "-dns")) {
			send_me_numeric(sptr, RPL_REHASHING, "DNS");
			sendto_ops("%s is rehashing DNS", parv[0]);
			restart_resolver();
		}
#ifdef USE_BLALLOC
		else if (!mycmp(parv[1], "-blockheap")) {
			send_me_numeric(sptr, RPL_REHASHING, "BLOCKHEAP");
			sendto_ops("%s is rehashing BLOCKHEAP", parv[0]);
			blockheap_cleanup();
		}
#endif
		else {
			send_me_notice(sptr, ":%s is an unknown rehash option - reloading everything",
				parv[1]);
			all = 1;
		}
	}
	else {
		all = 1;
	}
	if (all) {
		send_me_numeric(sptr, RPL_REHASHING, "ircd.conf");
		ircdlog(LOG_DEFAULT, "rehash requested by %s", get_client_name(sptr, FALSE));
		sendto_ops("%s has requested a rehash of ircd.conf", sptr->name);
		Internal.do_rehash = 1;
	}
	return 0;
}
