/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_restart.c: RESTART command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_restart.c,v 1.27 2004/03/07 16:58:30 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

static char buf[BUFSIZE];

extern void restart(char *);

Module MOD_HEADER(m_restart) = {
	"modules/commands/m_restart.so",
	"/RESTART command",
	5, "$Revision: 1.27 $"
};

int MOD_LOAD(m_restart)()
{
	register_command(&MOD_HEADER(m_restart), &CMD_RESTART, m_restart);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_restart)()
{
	return MOD_SUCCESS;
}

/*
 * m_restart
 *	parv[0] = sender prefix
 *	parv[1] = password
 */
int m_restart(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	if (!OPHasFlag(sptr, OFLAG_RESTART)) {
		send_me_numericNA(sptr, ERR_NOPRIVILEGES);
		return 0;
	}

	ircsprintf(buf, "Server restarting by request of %s (%s@%s).", sptr->name, sptr->username,
		MaskedHost(sptr));
	ircdlog(LOG_DEFAULT, "server restarting by request of %s!%s@%s", sptr->name, sptr->username,
		MaskedHost(sptr));

	restart(buf);

	return 0;
}
