/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_sakick.c: SAKICK command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_sakick.c,v 1.13 2004/03/07 16:58:30 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_sakick) = {
	"modules/commands/m_sakick.so",
	"/SAKICK command",
	5, "$Revision: 1.13 $"
};

int MOD_LOAD(m_sakick)()
{
	register_command(&MOD_HEADER(m_sakick), &CMD_SAKICK, m_sakick);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_sakick)()
{
	return MOD_SUCCESS;
}

/*
 * m_sakick
 *	parv[0] = sender prefix
 *	parv[1] = channel
 *	parv[2] = user
 *	parv[3] = reason
 */
int m_sakick(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aChannel *chptr;
	aClient *acptr;
	char *comment = (parc > 3 && *parv[3] != '\0') ? parv[3] : parv[0], *p;

	if (!IsPrivileged(sptr) || !HasMode(sptr, UMODE_SADMIN)) {
		send_me_numericNA(sptr, ERR_NOPRIVILEGES);
		return 0;
	}
	if (parc < 3) {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "SAKICK");
		return 0;
	}
	if ((p = strchr(parv[1], ',')) != NULL) {
		*p = '\0';
	}
	if ((p = strchr(parv[2], ',')) != NULL) {
		*p = '\0';
	}
	if ((chptr = find_channel(parv[1], NULL)) == NULL) {
		send_me_numeric(sptr, ERR_NOSUCHCHANNEL, parv[1]);
		return 0;
	}
	if ((acptr = find_chasing(sptr, parv[2], NULL, MSG_SAKICK)) == NULL) {
		return 0;
	}
	if (!IsMember(acptr, chptr)) {
		if (MyClient(sptr)) {
			send_me_numeric(sptr, ERR_USERNOTINCHANNEL, chptr->chname, acptr->name);
		}
		return 0;
	}
	if (strlen(comment) > TOPICLEN) {
		comment[TOPICLEN] = '\0';
	}

	sendto_realops("%s used SAKICK %s %s (%s)", sptr->name, chptr->chname, acptr->name, comment);
	sendto_serv_msg_butone(NULL, &me, NULL, &CMD_GLOBOPS, ":%s used SAKICK %s %s (%s)", sptr->name,
		chptr->chname, acptr->name, comment);
	ircdlog(LOG_OVERRIDE, "%s used SAKICK %s %s (%s)", get_client_name(sptr, FALSE),
		chptr->chname, acptr->name, comment);

	send_channel_msg_local(sptr, chptr, &CMD_KICK, "%s :%s", acptr->name, comment);
	sendto_serv_msg_butone(cptr, sptr, NULL, &CMD_KICK, "%s %s :%s", chptr->chname, acptr->name,
		comment);
	remove_user_from_channel(acptr, chptr);
	return 0;
}
