/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_samode.c: SAMODE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_samode.c,v 1.27 2004/05/13 17:43:51 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

static int (*p_do_chan_mode)(aClient *, aClient *, aChannel *, int, char **, int);

Module MOD_HEADER(m_samode) = {
	"modules/commands/m_samode.so",
	"/SAMODE command",
	6, "$Revision: 1.27 $"
};

int MOD_LOAD(m_samode)()
{
	if ((p_do_chan_mode = find_symbol(do_chan_mode, "modules/commands/m_mode.so")) == NULL) {
		ircdlog(LOG_ERROR, "m_samode.so: can't resolve symbol do_chan_mode");
		return MOD_FAILURE;
	}

	register_command(&MOD_HEADER(m_samode), &CMD_SAMODE, m_samode);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_samode)()
{
	return MOD_SUCCESS;
}

/*
 * m_samode
 *	parv[0] = sender prefix
 *	parv[1] = channel
 *	parv[2] = modes
 */
int m_samode(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aChannel *chptr;

	if (!IsPrivileged(sptr) || !HasMode(sptr, UMODE_SADMIN)) {
		send_me_numericNA(sptr, ERR_NOPRIVILEGES);
		return 0;
	}
	if (parc < 3) {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "SAMODE");
		return 0;
	}
	if ((chptr = find_channel(parv[1], NULL)) == NULL) {
		send_me_numeric(sptr, ERR_NOSUCHCHANNEL, parv[1]);
		return 0;
	}

	if ((*p_do_chan_mode)(cptr, &me, chptr, parc - 2, parv + 2, CPRIV_SADMIN) > 0
	  && MyClient(sptr)) {
		sendto_realops("%s used SAMODE %s (%s%s%s)", sptr->name, chptr->chname,
			modebuf, !BadPtr(parabuf) ? " " : "", !BadPtr(parabuf) ? parabuf : "");
		sendto_serv_msg_butone(NULL, &me, NULL, &CMD_GLOBOPS, ":%s used SAMODE %s (%s%s%s)",
			sptr->name, chptr->chname, modebuf, !BadPtr(parabuf) ? " " : "",
			!BadPtr(parabuf) ? parabuf : "");
		ircdlog(LOG_OVERRIDE, "%s used SAMODE %s (%s %s)", get_client_name(sptr, FALSE),
			chptr->chname, modebuf, !BadPtr(parabuf) ? parabuf : "<NOPARAS>");
	}
	return 0;
}
