/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_silence.c: SILENCE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_silence.c,v 1.18 2004/03/27 13:51:25 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_silence) = {
	"modules/commands/m_silence.so",
	"/SILCENCE command",
	5, "$Revision: 1.18 $"
};

int MOD_LOAD(m_silence)()
{
	register_command(&MOD_HEADER(m_silence), &CMD_SILENCE, m_silence);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_silence)()
{
	return MOD_SUCCESS;
}

/*
 * m_silence
 *	parv[0] = sender prefix
 *	If local client:
 *		parv[1] = mask (NULL sends the list)
 *	If remote client:
 *		parv[1] = nick to be silenced
 *		parv[2] = mask
 */
int m_silence(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr = NULL;
	SLink *lp;
	char c, *cp;

	if (MyClient(sptr)) {
		acptr = sptr;
		if (parc < 2 || *parv[1] == '\0' || (acptr = find_person(parv[1], NULL)) != NULL) {
			if (acptr->user == NULL) {
				return 0;
			}
			for (lp = acptr->user->silence; lp != NULL; lp = lp->next) {
				send_me_numeric(sptr, RPL_SILELIST, acptr->name, lp->value.cp);
			}
			send_me_numericNA(sptr, RPL_ENDOFSILELIST);
			return 0;
		}

		cp = parv[1];
		c = *cp;

		if (c == '-' || (c == '+')) {
			cp++;
		}
		else if (!(strchr(cp, '@') || strchr(cp, '.') || strchr(cp, '!') || strchr(cp, '*'))) {
			target_left(sptr, parv[1], MSG_SILENCE, NULL);
			return 0;
		}
		else {
			c = '+';
		}

		cp = pretty_mask(cp);
		if ((c == '-' && !del_silence(sptr, cp)) || (c != '-' && !add_silence(sptr, cp))) {
			sendto_prefix_one(sptr, sptr, ":%s SILENCE %c%s", sptr->name, c, cp);
			if (c == '-') {
				sendto_serv_msg_butone(cptr, sptr, NULL, &CMD_SILENCE, "* -%s", cp);
			}
		}
	}
	else if (parc < 3 || *parv[2] == '\0') {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "SILENCE");
	}
	else if ((c = *parv[2]) == '-' || (acptr = find_person(parv[1], NULL)) != NULL) {
		if (c == '-') {
			if (!del_silence(sptr, parv[2] + 1)) {
				sendto_serv_msg_butone(cptr, sptr, NULL, &CMD_SILENCE, "%s %s",
					parv[1], parv[2]);
			}
		}
		else {
			add_silence(sptr, parv[2]);
			if (!MyClient(acptr)) {
				sendto_one_client(acptr, sptr, &CMD_SILENCE, "%s", parv[2]);
			}
		}
	}
	return 0;
}
