/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_sqline.c: SQLINE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_sqline.c,v 1.25 2004/03/07 16:58:30 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

extern char *collapse(char *);

Module MOD_HEADER(m_sqline) = {
	"modules/commands/m_sqline.so",
	"/SQLINE command",
	5, "$Revision: 1.25 $"
};

int MOD_LOAD(m_sqline)()
{
	register_command(&MOD_HEADER(m_sqline), &CMD_SQLINE, m_sqline);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_sqline)()
{
	return MOD_SUCCESS;
}

/*
 * m_sqline
 *	parv[0] = sender prefix
 *	parv[1] = nickname
 *	parv[2] = reason
 */
int m_sqline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	ConfigItem_ban *ban;
	char *reason = (parc > 2 && *parv[1] != '\0') ? parv[2] : "reserved";

	if (!(IsServer(sptr) || IsULine(sptr))) {
		return 0;
	}
	if (parc < 2) {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "SQLINE");
		return 0;
	}

	parv[1] = collapse(parv[1]);
	if ((ban = find_ban(BAN_NICKNAME, parv[1])) == NULL) {
		ban = (ConfigItem_ban *)MyMallocX(sizeof(ConfigItem_ban));
		ban->item.temp = 0;
		ban->flags |= (BAN_NICKNAME|BAN_SVS);
		DupString(ban->mask, parv[1]);
		DupString(ban->reason, reason);
		dlink_add(&conf_ban_list, ban);
	}
	sendto_serv_msg_butone(cptr, sptr, NULL, &CMD_SQLINE, "%s :%s", parv[1], parv[2]);
	return 0;
}
