/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_svskill.c: SVSKILL command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_svskill.c,v 1.18 2004/03/07 16:58:30 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_svskill) = {
	"modules/commands/m_svskill.so",
	"/SVSKILL command",
	5, "$Revision: 1.18 $"
};

int MOD_LOAD(m_svskill)()
{
	register_command(&MOD_HEADER(m_svskill), &CMD_SVSKILL, m_svskill);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_svskill)()
{
	return MOD_SUCCESS;
}

/*
 * m_svskill
 *	parv[0] = sender prefix
 *	parv[1] = target
 *	parv[2] = target stamp
 *	parv[3] = message
 */
int m_svskill(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr;
	char *comment, reason[TOPICLEN + 1];
	time_t ts = 0;

	if (!IsULine(sptr)) {
		return 0;
	}
	if (parc > 3) {
		comment = (*parv[3] != '\0') ? parv[3] : parv[0];
		ts = atol(parv[2]);
	}
	else {
		comment = (parc > 2 && *parv[2] != '\0') ? parv[2] : parv[0];
	}

	if ((acptr = find_person_target(parv[1])) != NULL && (!ts || ts == acptr->tsinfo)) {
		if (MyClient(acptr)) {
			strcpy(reason, "SVSKilled: ");
			strncpy(reason + 11, comment, TOPICLEN - 11);
			reason[TOPICLEN] = '\0';
			ircdlog(LOG_KILL, "SVSKILL for %s by %s", get_client_name(acptr, FALSE),
				get_client_name(cptr, FALSE));
			exit_client(acptr, acptr, sptr, reason);
			return (acptr == cptr) ?  FLUSH_BUFFER : 0;
		}
		if (acptr->from == cptr) {
			sendto_realops_lev(DEBUG_LEV, "Received wrong-direction SVSKILL for %s (behined "
				"%s) from %s", acptr->name, cptr->name, get_client_name(sptr, HIDE_IP));
		}
		else if (ts && !CapID(acptr->from)) {
			sendto_one_client_nopostfix(acptr, sptr, &CMD_SVSKILL, "%s %ld :%s", parv[1],
				ts, comment);
		}
		else {
			sendto_one_client_real(acptr->from, acptr, sptr, &CMD_SVSKILL, ":%s", comment);
		}
	}
	return 0;
}
