/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_topic.c: TOPIC command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_topic.c,v 1.20 2004/04/09 03:02:16 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_topic) = {
	"modules/commands/m_topic.so",
	"/TOPIC command",
	5, "$Revision: 1.20 $"
};

int MOD_LOAD(m_topic)()
{
	register_command(&MOD_HEADER(m_topic), &CMD_TOPIC, m_topic);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_topic)()
{
	return MOD_SUCCESS;
}

/*
 * m_topic
 *	parv[0] = sender prefix
 *	parv[1] = channel name
 */
int m_topic(aClient *cptr, aClient *sptr, int parc, char *parv[])	
{
	aChannel *chptr = NULL;
	char *topic = NULL, *name, *tnick = sptr->name;
	time_t ts = timeofday;
	int member;

	if (parc < 2 || *parv[1] == '\0') {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "TOPIC");
		return 0;
	}

	name = parv[1];
	if ((chptr = find_channel(name, NULL)) == NULL) {
		send_me_numeric(sptr, ERR_NOSUCHCHANNEL, name);
		return 0;
	}

	member = IsMember(sptr, chptr);
	if (parc == 2) {
		if (!member) {
			send_me_numeric(sptr, ERR_NOTONCHANNEL, name);
			return 0;
		}

		if (*chptr->topic == '\0') {
			send_me_numeric(sptr, RPL_NOTOPIC, chptr->chname);
		}
		else {
			send_me_numeric(sptr, RPL_TOPIC, chptr->chname, chptr->topic);
			send_me_numeric(sptr, RPL_TOPICWHOTIME, chptr->chname,
				chptr->topic_nick, chptr->topic_time);
		}
		return 0;
	}
	if (!member && !IsServer(sptr) && !IsULine(sptr)) {
		send_me_numeric(sptr, ERR_NOTONCHANNEL, name);
		return 0;
	}
	if (parc > 3 && (!MyConnect(sptr) || IsULine(sptr) || IsServer(sptr))) {
		topic = parc > 4 ? parv[4] : "";
		tnick = parv[2];
		ts = atoi(parv[3]);
	}
	else {
		topic = parv[2];
	}
	if (((!(chptr->mode.mode & CMODE_TOPICLIMIT) || is_chanop(sptr, chptr) || is_halfop(sptr, chptr))
	  || IsULine(sptr) || IsServer(sptr))) {
		if (IsServer(sptr) && !IsULine(sptr) && chptr->topic_time >= ts && *chptr->topic != '\0') {
			return 0;
		}

		strncpyzt(chptr->topic, topic, TOPICLEN + 1);
		strcpy(chptr->topic_nick, tnick);
		chptr->topic_time = ts;

		sendto_serv_msg_butone(cptr, sptr, NULL, &CMD_TOPIC, "%s %s %lu :%s", chptr->chname,
			chptr->topic_nick, chptr->topic_time, chptr->topic);
		send_channel_msg_local(sptr, chptr, &CMD_TOPIC, ":%s", chptr->topic);
	}
	else {
		send_me_numeric(sptr, ERR_CHANOPRIVSNEEDED, chptr->chname);
	}
	return 0;
}
