/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_unkline.c: UNKLINE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_unkline.c,v 1.25 2004/04/05 00:54:21 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_unkline) = {
	"modules/commands/m_unkline.so",
	"/UNKLINE command",
	5, "$Revision: 1.25 $"
};

int MOD_LOAD(m_unkline)()
{
	register_command(&MOD_HEADER(m_unkline), &CMD_UNKLINE, m_unkline);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_unkline)()
{
	return MOD_SUCCESS;
}

/*
 * m_unkline
 *	parv[0] = sender prefix
 *	parv[1] = mask
 */
int m_unkline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *user, *host, *userhost;
	dlink_node *node, *next = NULL;
	ConfigItem_ban *ban;
	int i = 0;

	if (!OPHasFlag(sptr, OFLAG_UNKLINE)) {
		send_me_numericNA(sptr, ERR_NOPRIVILEGES);
		return 0;
	}
	if (parc < 2 || BadPtr(parv[1])) {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "UNKLINE");
		return 0;
	}
	if ((host = strchr(parv[1], '@')) != NULL) {
		user = parv[1];
		*host++ = '\0';
	}
	else {
		user = "*";
		host = parv[1];
	}

	userhost = make_user_host(user, host);
	DLINK_FOREACH_SAFE_DATA(conf_ban_list.head, node, next, ban, ConfigItem_ban) {
		if ((ban->flags & (BAN_USERHOST|BAN_SVS)) != BAN_USERHOST || ban->in_file) {
			continue;
		}
		if (!match(ban->mask, userhost) || !match(userhost, ban->mask)) {
			ircdlog(LOG_BAN, "kline on %s removed by %s as %s", ban->mask,
				get_client_name(cptr, SHOW_IP),	userhost);
			dlink_del(&conf_ban_list, NULL, node);
			free_ban(ban);
			i++;
		}
	}
	if (i) {
		sendto_realops("%s (%s@%s) has removed %d K:line%s matching %s",
			sptr->name, sptr->username, MaskedHost(sptr), i, i == 1 ? "" : "s", userhost);
	}
	else {
		send_me_notice(sptr, ":%s is not currently on the K:line list, or is not removeable",
			userhost);
	}
	return 0;
}
