/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_user.c: USER command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_user.c,v 1.36 2004/05/08 16:12:02 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_user) = {
	"modules/commands/m_user.so",
	"/USER command",
	6, "$Revision: 1.36 $"
};

int MOD_LOAD(m_user)()
{
	register_command(&MOD_HEADER(m_user), &CMD_USER, m_user);
	MOD_SET_FLAG(&MOD_HEADER(m_user), MOD_FLAG_PERM);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_user)()
{
	return MOD_SUCCESS;
}

/*
 * m_user
 *	parv[0] = sender prefix
 *	parv[1] = username
 *	parv[2] = client hostname (used only from other servers)
 *	parv[3] = server hostname (used only from other servers)
 *	parv[4] = users real name info
 */
int m_user(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *username, *host, *server, *realname;
	ConfigItem_ban *ban;

	if (sptr->localClient->listener != NULL) {
		if (sptr->localClient->listener->flags & LISTEN_SERVERONLY) {
			sendto_realops_lev(REJ_LEV, "Blocking unknown connection on server-only port %d",
				cptr->localClient->listener->port);
			return exit_client(cptr, sptr, sptr, "This port is for servers only");
		}
	}

	if (parc > 2 && (username = strchr(parv[1], '@')) != NULL) {
		*username = '\0';
	}
	if (parc < 5 || *parv[1] == '\0' || *parv[2] == '\0' || *parv[3] == '\0' || *parv[4] == '\0') {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "USER");
		if (IsServer(cptr)) {
			sendto_realops_lev(DEBUG_LEV, "Bad USER param count for %s from %s", parv[0],
				get_client_name(cptr, FALSE));
		}
		else {
			return 0;
		}
	}

	username = (parc < 2 || BadPtr(parv[1])) ? "<bad-boy>" : parv[1];
	host = (parc < 3 || BadPtr(parv[2])) ? "<nohost>" : parv[2];
	server = (parc < 4 || BadPtr(parv[3])) ? "<noserver>" : parv[3];
	realname = (parc < 5 || BadPtr(parv[4])) ? "<bad-realname>" : parv[4];

	if ((ban = find_ban(BAN_GCOS, realname)) != NULL) {
		return conf_do_gline(ban, cptr, sptr);
	}

	return do_user(parv[0], cptr, sptr, username, host, server, 0, 0, realname);
}
