/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/packet.c: internet packet handling
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: packet.c,v 1.27 2004/04/08 21:24:27 rageircd Exp $
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "msg.h"
#include "h.h"
#include "ssl.h"
#include "zlink.h"
#include "memory.h"

int serv_dopacket(aClient *cptr, char *buffer, int length)
{
	char *ch1, *ch2, *cptrbuf, *nbuf = NULL;
	int nlen;

	ASSERT(cptr != NULL);
	ASSERT(cptr->localClient != NULL);

	cptrbuf = cptr->localClient->buffer;

#ifdef USE_OPENSSL
	if (InputRC4(cptr)) {
		rc4_process_stream(cptr->serv->rc4_in, buffer, length);
	}
#endif

	cptr->localClient->receiveB += length;
	if (cptr->localClient->receiveB & 0x0400) {
		cptr->localClient->receiveK += (cptr->localClient->receiveB >> 10);
		cptr->localClient->receiveB &= 0x03ff;
	}

	me.localClient->receiveB += length;
	if (me.localClient->receiveB & 0x0400) {
		me.localClient->receiveK += (me.localClient->receiveB >> 10);
		me.localClient->receiveB &= 0x03ff;
	}

	if (cptr->localClient->listener != NULL) {
		cptr->localClient->listener->receiveB += length;
		if (cptr->localClient->listener->receiveB & 0x0400) {
			cptr->localClient->listener->receiveK += (cptr->localClient->listener->receiveB >> 10);
			cptr->localClient->listener->receiveB &= 0x03ff;
		}
	}

		zcontinue:
	ch1 = cptrbuf + cptr->localClient->count;
	ch2 = buffer;

	if (InputZIP(cptr)) {
		int err;

		ch2 = zip_input(cptr->serv->zip_in, ch2, &length, &err, &nbuf, &nlen);
		if (length == -1) {
			sendto_realops("ZipIn error for %s: (%d) %s", cptr->name, err, ch2);
			return exit_client(cptr, cptr, &me, "Fatal error in zip_input()");
		}
	}
	while (--length >= 0) {
		char g = (*ch1 = *ch2++);
		if (g < '\16' && (g == '\n' || g == '\r')) {
			if (ch1 == cptrbuf) {
				continue;
			}

			*ch1 = '\0';

			me.localClient->receiveM++;
			cptr->localClient->receiveM++;
			if (cptr->localClient->listener != NULL) {
				cptr->localClient->listener->receiveM++;
			}

			cptr->localClient->count = 0;

			switch (serv_parse(cptr, cptr->localClient->buffer, ch1)) {
				case FLUSH_BUFFER:
					return FLUSH_BUFFER;
				case ZIP_NEXT_BUFFER:
					if (length) {
						int err;

						ch2 = zip_input(cptr->serv->zip_in, ch2, &length, &err, &nbuf, &nlen);
						if (length == -1) {
							sendto_realops("ZipIn error for %s: (%d) %s", cptr->name,
								err, ch2);
							return exit_client(cptr, cptr, &me, "Fatal error in zip_input()");
						}
					}
					break;
#ifdef USE_OPENSSL
				case RC4_NEXT_BUFFER:
					if (length) {
						rc4_process_stream(cptr->serv->rc4_in, ch2, length);
					}
					break;
#endif
				default:
					break;
			}

			if (DeadSocket(cptr)) {
				return exit_client(cptr, cptr, &me, SendQEx(cptr) ? "SendQ Exceeded" : "Dead Socket");
			}

			ch1 = cptrbuf;
		}
		else if (ch1 < cptrbuf + (BUFSIZE - 1)) {
			ch1++;
		}
	}

	cptr->localClient->count = ch1 - cptrbuf;

	if (nbuf != NULL) {
		buffer = nbuf;
		length = nlen;
		nbuf = NULL;
		goto zcontinue;
	}

	return 0;
}

int user_dopacket(aClient *cptr, char *buffer, int length)
{
	ASSERT(cptr != NULL);
	ASSERT(cptr->localClient != NULL);

	strncpy(cptr->localClient->buffer, buffer, BUFSIZE);
	length = strlen(cptr->localClient->buffer);

	me.localClient->receiveM++;
	cptr->localClient->receiveM++;
	if (cptr->localClient->listener != NULL) {
		cptr->localClient->listener->receiveM++;
	}

	cptr->localClient->receiveB += length;
	if (cptr->localClient->receiveB & 0x0400) {
		cptr->localClient->receiveK += (cptr->localClient->receiveB >> 10);
		cptr->localClient->receiveB &= 0x03ff;
	}

	me.localClient->receiveB += length;
	if (me.localClient->receiveB > 1023) {
		me.localClient->receiveK += (me.localClient->receiveB >> 10);
		me.localClient->receiveB &= 0x03ff;
	}

	if (cptr->localClient->listener != NULL) {
		cptr->localClient->listener->receiveB += length;
		if (cptr->localClient->listener->receiveB & 0x0400) {
			cptr->localClient->listener->receiveK += (cptr->localClient->listener->receiveB >> 10);
			cptr->localClient->listener->receiveB &= 0x03ff;
		}
	}

	cptr->localClient->count = 0;

	if (user_parse(cptr, cptr->localClient->buffer, (cptr->localClient->buffer + length)) == FLUSH_BUFFER) {
		return FLUSH_BUFFER;
	}

	return 1;
}
