/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/scache.c: server cache
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: scache.c,v 1.7 2004/03/27 01:17:47 rageircd Exp $
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "h.h"
#include "memory.h"

static ServerCache *server_cache[SCACHE_HASH_SIZE];

void clear_scache_hash_table()
{
	memset(server_cache, '\0', SCACHE_HASH_SIZE);
}

static int serv_hash(char *str)
{
	int sindex = 0;

	while (*str != '\0') {
		sindex += tolowertab[(unsigned char)*str++];
	}

	return (sindex % SCACHE_HASH_SIZE);
}

char *find_or_add(char *name)
{
	int sindex = serv_hash(name);
	ServerCache *serv;

	for (serv = server_cache[sindex]; serv != NULL; serv = serv->next) {
		if (!irccmp(serv->name, name)) {
			return serv->name;
		}
	}

	serv = (ServerCache *)MyMalloc(sizeof(ServerCache));
	strncpyzt(serv->name, name, HOSTLEN + 1);
	serv->next = server_cache[sindex];
	server_cache[sindex] = serv;

	return serv->name;
}

void count_scache_memory(int *scache_cnt, unsigned long *scache_mem)
{
	ServerCache *serv = NULL;
	int sindex, cnt = 0;
	unsigned long mem = 0;

	for (sindex = 0; sindex < SCACHE_HASH_SIZE; sindex++) {
		for (serv = server_cache[sindex]; serv != NULL; serv = serv->next) {
			cnt++;
			mem += strlen(serv->name);
		}
	}

	mem += (cnt * sizeof(ServerCache));

	*scache_cnt = cnt;
	*scache_mem = mem;
}

void list_scache(aClient *cptr)
{
	ServerCache *serv;
	int sindex;

	for (sindex = 0; sindex < SCACHE_HASH_SIZE; sindex++) {
		for (serv = server_cache[sindex]; serv != NULL; serv = serv->next) {
			ASSERT(!BadPtr(serv->name));
			send_me_notice(cptr, ":%s", serv->name);
		}
	}
}
